@@ -86,9 +86,9 @@ cc_applications_row_new (GAppInfo *info)
 
   icon = g_app_info_get_icon (info);
   if (icon != NULL)
-    gtk_image_set_from_gicon (GTK_IMAGE (self->image), g_app_info_get_icon (info), GTK_ICON_SIZE_BUTTON);
+    gtk_image_set_from_gicon (GTK_IMAGE (self->image), g_app_info_get_icon (info));
   else
-    gtk_image_set_from_icon_name (GTK_IMAGE (self->image), "application-x-executable", GTK_ICON_SIZE_BUTTON);
+    gtk_image_set_from_icon_name (GTK_IMAGE (self->image), "application-x-executable");
 
   gtk_label_set_label (GTK_LABEL (self->label), g_app_info_get_display_name (info));
 
