@@ -17,30 +17,26 @@
  *
  */
 
-#ifndef __CC_BACKGROUND_ITEM_H
-#define __CC_BACKGROUND_ITEM_H
+#pragma once
 
 #include <glib-object.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <libgnome-desktop/gnome-desktop-thumbnail.h>
 #include <gdesktop-enums.h>
-#include <libgnome-desktop/gnome-bg.h>
+#include <gnome-bg/gnome-bg.h>
 
 G_BEGIN_DECLS
 
-#define CC_TYPE_BACKGROUND_ITEM         (cc_background_item_get_type ())
-#define CC_BACKGROUND_ITEM(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), CC_TYPE_BACKGROUND_ITEM, CcBackgroundItem))
-#define CC_BACKGROUND_ITEM_CLASS(k)     (G_TYPE_CHECK_CLASS_CAST((k), CC_TYPE_BACKGROUND_ITEM, CcBackgroundItemClass))
-#define CC_IS_BACKGROUND_ITEM(o)        (G_TYPE_CHECK_INSTANCE_TYPE ((o), CC_TYPE_BACKGROUND_ITEM))
-#define CC_IS_BACKGROUND_ITEM_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), CC_TYPE_BACKGROUND_ITEM))
-#define CC_BACKGROUND_ITEM_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), CC_TYPE_BACKGROUND_ITEM, CcBackgroundItemClass))
+#define CC_TYPE_BACKGROUND_ITEM (cc_background_item_get_type ())
+G_DECLARE_FINAL_TYPE (CcBackgroundItem, cc_background_item, CC, BACKGROUND_ITEM, GObject)
 
 typedef enum {
 	CC_BACKGROUND_ITEM_HAS_SHADING   = 1 << 0,
 	CC_BACKGROUND_ITEM_HAS_PLACEMENT = 1 << 1,
 	CC_BACKGROUND_ITEM_HAS_PCOLOR    = 1 << 2,
 	CC_BACKGROUND_ITEM_HAS_SCOLOR    = 1 << 3,
-	CC_BACKGROUND_ITEM_HAS_URI       = 1 << 4
+	CC_BACKGROUND_ITEM_HAS_URI       = 1 << 4,
+	CC_BACKGROUND_ITEM_HAS_URI_DARK  = 1 << 5
 } CcBackgroundItemFlags;
 
 #define CC_BACKGROUND_ITEM_HAS_ALL (CC_BACKGROUND_ITEM_HAS_SHADING &	\
@@ -49,43 +45,32 @@ typedef enum {
 				    CC_BACKGROUND_ITEM_HAS_SCOLOR &	\
 				    CC_BACKGROUND_ITEM_HAS_FNAME)
 
-typedef struct CcBackgroundItemPrivate CcBackgroundItemPrivate;
-
-typedef struct
-{
-        GObject                  parent;
-        CcBackgroundItemPrivate *priv;
-} CcBackgroundItem;
-
-typedef struct
-{
-        GObjectClass   parent_class;
-} CcBackgroundItemClass;
-
-GType              cc_background_item_get_type (void);
-
 CcBackgroundItem * cc_background_item_new                 (const char                   *uri);
 CcBackgroundItem * cc_background_item_copy                (CcBackgroundItem             *item);
-gboolean           cc_background_item_load                (CcBackgroundItem             *item,
-							   GFileInfo                    *info);
+gboolean           cc_background_item_load                (CcBackgroundItem *item,
+                                                           GFileInfo        *info);
 gboolean           cc_background_item_changes_with_time   (CcBackgroundItem             *item);
+gboolean           cc_background_item_has_dark_version    (CcBackgroundItem             *item);
 
 GdkPixbuf *        cc_background_item_get_thumbnail       (CcBackgroundItem             *item,
                                                            GnomeDesktopThumbnailFactory *thumbs,
                                                            int                           width,
                                                            int                           height,
-                                                           int                           scale_factor);
+                                                           int                           scale_factor,
+                                                           gboolean                      dark);
 GdkPixbuf *        cc_background_item_get_frame_thumbnail (CcBackgroundItem             *item,
                                                            GnomeDesktopThumbnailFactory *thumbs,
                                                            int                           width,
                                                            int                           height,
                                                            int                           scale_factor,
                                                            int                           frame,
-                                                           gboolean                      force_size);
+                                                           gboolean                      force_size,
+                                                           gboolean                      dark);
 
 GDesktopBackgroundStyle   cc_background_item_get_placement  (CcBackgroundItem *item);
 GDesktopBackgroundShading cc_background_item_get_shading    (CcBackgroundItem *item);
 const char *              cc_background_item_get_uri        (CcBackgroundItem *item);
+const char *              cc_background_item_get_uri_dark   (CcBackgroundItem *item);
 const char *              cc_background_item_get_source_url (CcBackgroundItem *item);
 const char *              cc_background_item_get_source_xml (CcBackgroundItem *item);
 CcBackgroundItemFlags     cc_background_item_get_flags      (CcBackgroundItem *item);
@@ -101,5 +86,3 @@ gboolean                  cc_background_item_compare        (CcBackgroundItem *s
 void                      cc_background_item_dump           (CcBackgroundItem *item);
 
 G_END_DECLS
-
-#endif /* __CC_BACKGROUND_ITEM_H */
