@@ -30,10 +30,10 @@
 #include "gsd-input-helper.h"
 
 #ifdef GDK_WINDOWING_X11
-#include <gdk/gdkx.h>
+#include <gdk/x11/gdkx.h>
 #endif
 #ifdef GDK_WINDOWING_WAYLAND
-#include <gdk/gdkwayland.h>
+#include <gdk/wayland/gdkwayland.h>
 #endif
 
 typedef struct
@@ -400,14 +400,10 @@ device_is_evdev (GUdevDevice *device)
 }
 
 static GsdDevice *
-create_device (GUdevDevice *udev_device)
+create_device (GUdevDevice *udev_device, GUdevDevice *parent)
 {
 	const gchar *vendor, *product, *name, *group;
 	guint width, height;
-	g_autoptr(GUdevDevice) parent = NULL;
-
-	parent = g_udev_device_get_parent (udev_device);
-	g_assert (parent != NULL);
 
 	name = g_udev_device_get_sysfs_attr (parent, "name");
 	vendor = g_udev_device_get_property (udev_device, "ID_VENDOR_ID");
@@ -440,7 +436,7 @@ add_device (GsdDeviceManager *manager,
 	    GUdevDevice	     *udev_device)
 {
         GsdDeviceManagerPrivate *priv = gsd_device_manager_get_instance_private (manager);
-	GUdevDevice *parent;
+	g_autoptr(GUdevDevice) parent = NULL;
 	GsdDevice *device;
 	const gchar *syspath;
 
@@ -449,7 +445,7 @@ add_device (GsdDeviceManager *manager,
 	if (!parent)
 		return;
 
-	device = create_device (udev_device);
+	device = create_device (udev_device, parent);
 	syspath = g_udev_device_get_sysfs_path (udev_device);
 	g_hash_table_insert (priv->devices, g_strdup (syspath), device);
 	g_signal_emit_by_name (manager, "device-added", device);
@@ -522,18 +518,18 @@ GsdDeviceManager *
 gsd_device_manager_get (void)
 {
 	GsdDeviceManager *manager;
-	GdkScreen *screen;
+	GdkDisplay *display;
 
-	screen = gdk_screen_get_default ();
-	g_return_val_if_fail (screen != NULL, NULL);
+	display = gdk_display_get_default ();
+	g_return_val_if_fail (display != NULL, NULL);
 
-	manager = g_object_get_data (G_OBJECT (screen), "gsd-device-manager-data");
+	manager = g_object_get_data (G_OBJECT (display), "gsd-device-manager-data");
 
 	if (!manager) {
                 manager = g_object_new (GSD_TYPE_DEVICE_MANAGER,
                                         NULL);
 
-		g_object_set_data_full (G_OBJECT (screen), "gsd-device-manager-data",
+		g_object_set_data_full (G_OBJECT (display), "gsd-device-manager-data",
 					manager, (GDestroyNotify) g_object_unref);
 	}
 
