@@ -18,10 +18,8 @@
  * Author: Matthias Clasen <mclasen@redhat.com>
  */
 
-#include "list-box-helper.h"
 #include "cc-diagnostics-panel.h"
 #include "cc-diagnostics-resources.h"
-#include "cc-os-release.h"
 #include "cc-util.h"
 #include "shell/cc-application.h"
 
@@ -30,13 +28,12 @@
 
 struct _CcDiagnosticsPanel
 {
-  CcPanel     parent_instance;
+  CcPanel              parent_instance;
 
-  GtkLabel   *diagnostics_explanation_label;
-  GtkListBox *diagnostics_list_box;
-  GtkSwitch   *abrt_switch;
+  AdwPreferencesGroup *diagnostics_group;
+  GtkSwitch           *abrt_switch;
 
-  GSettings  *privacy_settings;
+  GSettings           *privacy_settings;
 };
 
 CC_PANEL_REGISTER (CcDiagnosticsPanel, cc_diagnostics_panel)
@@ -108,8 +105,7 @@ cc_diagnostics_panel_class_init (CcDiagnosticsPanelClass *klass)
 
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/control-center/diagnostics/cc-diagnostics-panel.ui");
 
-  gtk_widget_class_bind_template_child (widget_class, CcDiagnosticsPanel, diagnostics_explanation_label);
-  gtk_widget_class_bind_template_child (widget_class, CcDiagnosticsPanel, diagnostics_list_box);
+  gtk_widget_class_bind_template_child (widget_class, CcDiagnosticsPanel, diagnostics_group);
   gtk_widget_class_bind_template_child (widget_class, CcDiagnosticsPanel, abrt_switch);
 }
 
@@ -125,18 +121,16 @@ cc_diagnostics_panel_init (CcDiagnosticsPanel *self)
 
   gtk_widget_init_template (GTK_WIDGET (self));
 
-  gtk_list_box_set_header_func (self->diagnostics_list_box,
-                                cc_list_box_update_header_func,
-                                NULL, NULL);
-
   self->privacy_settings = g_settings_new ("org.gnome.desktop.privacy");
 
   g_settings_bind (self->privacy_settings, "report-technical-problems",
                    self->abrt_switch, "active",
                    G_SETTINGS_BIND_DEFAULT);
 
-  os_name = cc_os_release_get_value ("NAME");
-  url = cc_os_release_get_value ("PRIVACY_POLICY_URL");
+  os_name = g_get_os_info (G_OS_INFO_KEY_NAME);
+  if (!os_name)
+    os_name = g_strdup ("GNOME");
+  url = g_get_os_info (G_OS_INFO_KEY_PRIVACY_POLICY_URL);
   if (!url)
     url = g_strdup ("http://www.gnome.org/privacy-policy");
   /* translators: Text used in link to privacy policy */
@@ -145,5 +139,5 @@ cc_diagnostics_panel_init (CcDiagnosticsPanel *self)
   msg = g_strdup_printf (_("Sending reports of technical problems helps us improve %s. Reports "
                            "are sent anonymously and are scrubbed of personal data. %s"),
                          os_name, link);
-  gtk_label_set_markup (self->diagnostics_explanation_label, msg);
+  adw_preferences_group_set_description (self->diagnostics_group, msg);
 }
