@@ -74,7 +74,6 @@ struct _CcDisplayPanel
   guint           focus_id;
 
   CcNightLightPage *night_light_page;
-  GtkDialog *night_light_dialog;
   GtkLabel *night_light_state_label;
 
   UpClient *up_client;
@@ -92,11 +91,11 @@ struct _CcDisplayPanel
 
   GtkWidget      *display_settings_disabled_group;
 
-  GtkWidget      *arrangement_group;
+  GtkWidget      *arrangement_row;
   AdwBin         *arrangement_bin;
   GtkToggleButton *config_type_join;
   GtkToggleButton *config_type_mirror;
-  GtkWidget      *config_type_switcher_row;
+  GtkWidget      *display_multiple_displays;
   AdwBin         *display_settings_bin;
   GtkWidget      *display_settings_group;
   AdwWindowTitle *display_settings_title_widget;
@@ -437,8 +436,6 @@ cc_display_panel_dispose (GObject *object)
 
   g_clear_object (&self->shell_proxy);
 
-  g_clear_pointer ((GtkWindow **) &self->night_light_dialog, gtk_window_destroy);
-
   g_signal_handlers_disconnect_by_data (toplevel, self);
 
   G_OBJECT_CLASS (cc_display_panel_parent_class)->dispose (object);
@@ -486,27 +483,6 @@ on_config_type_toggled_cb (CcDisplayPanel *panel,
   config_ensure_of_type (panel, type);
 }
 
-static void
-on_night_light_list_box_row_activated_cb (CcDisplayPanel *panel)
-{
-  GtkWindow *toplevel;
-  toplevel = GTK_WINDOW (cc_shell_get_toplevel (cc_panel_get_shell (CC_PANEL (panel))));
-
-  if (!panel->night_light_dialog)
-    {
-      GtkWidget *content_area;
-
-      panel->night_light_dialog = (GtkDialog *)gtk_dialog_new ();
-
-      content_area = gtk_dialog_get_content_area (panel->night_light_dialog);
-      gtk_box_append (GTK_BOX (content_area), GTK_WIDGET (panel->night_light_page));
-      gtk_widget_show (GTK_WIDGET (panel->night_light_page));
-    }
-
-  gtk_window_set_transient_for (GTK_WINDOW (panel->night_light_dialog), toplevel);
-  gtk_window_present (GTK_WINDOW (panel->night_light_dialog));
-}
-
 static void
 on_night_light_enabled_changed_cb (GSettings      *settings,
                                    const gchar    *key,
@@ -607,10 +583,10 @@ cc_display_panel_class_init (CcDisplayPanelClass *klass)
   gtk_widget_class_bind_template_child (widget_class, CcDisplayPanel, apply_titlebar);
   gtk_widget_class_bind_template_child (widget_class, CcDisplayPanel, apply_titlebar_title_widget);
   gtk_widget_class_bind_template_child (widget_class, CcDisplayPanel, display_settings_disabled_group);
-  gtk_widget_class_bind_template_child (widget_class, CcDisplayPanel, arrangement_group);
   gtk_widget_class_bind_template_child (widget_class, CcDisplayPanel, arrangement_bin);
+  gtk_widget_class_bind_template_child (widget_class, CcDisplayPanel, arrangement_row);
   gtk_widget_class_bind_template_child (widget_class, CcDisplayPanel, cancel_button);
-  gtk_widget_class_bind_template_child (widget_class, CcDisplayPanel, config_type_switcher_row);
+  gtk_widget_class_bind_template_child (widget_class, CcDisplayPanel, display_multiple_displays);
   gtk_widget_class_bind_template_child (widget_class, CcDisplayPanel, config_type_join);
   gtk_widget_class_bind_template_child (widget_class, CcDisplayPanel, config_type_mirror);
   gtk_widget_class_bind_template_child (widget_class, CcDisplayPanel, display_settings_bin);
@@ -627,7 +603,6 @@ cc_display_panel_class_init (CcDisplayPanelClass *klass)
   gtk_widget_class_bind_template_callback (widget_class, apply_current_configuration);
   gtk_widget_class_bind_template_callback (widget_class, on_back_button_clicked_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_config_type_toggled_cb);
-  gtk_widget_class_bind_template_callback (widget_class, on_night_light_list_box_row_activated_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_night_light_row_activated_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_primary_display_selected_item_changed_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_screen_changed);
@@ -692,6 +667,7 @@ add_display_row (CcDisplayPanel   *self,
   number_string = g_strdup_printf ("%d", cc_display_monitor_get_ui_number (monitor));
   number_label = gtk_label_new (number_string);
   gtk_widget_set_valign (number_label, GTK_ALIGN_CENTER);
+  gtk_widget_set_halign (number_label, GTK_ALIGN_CENTER);
   gtk_widget_add_css_class (number_label, "monitor-label");
   adw_action_row_add_prefix (ADW_ACTION_ROW (row), number_label);
 
@@ -752,7 +728,7 @@ rebuild_ui (CcDisplayPanel *panel)
     {
       gtk_widget_set_visible (panel->display_settings_disabled_group, TRUE);
       gtk_widget_set_visible (panel->display_settings_group, FALSE);
-      gtk_widget_set_visible (panel->arrangement_group, FALSE);
+      gtk_widget_set_visible (panel->arrangement_row, FALSE);
       return;
     }
 
@@ -822,11 +798,14 @@ rebuild_ui (CcDisplayPanel *panel)
       if (type > CC_DISPLAY_CONFIG_LAST_VALID)
         type = CC_DISPLAY_CONFIG_JOIN;
 
-      gtk_widget_set_visible (panel->display_settings_group, TRUE);
-      gtk_widget_set_visible (panel->config_type_switcher_row, TRUE);
-      gtk_widget_set_visible (panel->arrangement_group, type == CC_DISPLAY_CONFIG_JOIN);
+      gtk_widget_set_visible (panel->display_settings_group, type == CC_DISPLAY_CONFIG_JOIN);
+      gtk_widget_set_visible (panel->display_multiple_displays, TRUE);
+      gtk_widget_set_visible (panel->arrangement_row, type == CC_DISPLAY_CONFIG_JOIN);
 
-      move_display_settings_to_separate_page (panel);
+      if (type == CC_DISPLAY_CONFIG_CLONE)
+        move_display_settings_to_main_page (panel);
+      else
+        move_display_settings_to_separate_page (panel);
     }
   else if (n_usable_outputs > 1)
     {
@@ -834,8 +813,8 @@ rebuild_ui (CcDisplayPanel *panel)
        * and we always show the arrangement widget.
        */
       gtk_widget_set_visible (panel->display_settings_group, TRUE);
-      gtk_widget_set_visible (panel->config_type_switcher_row, FALSE);
-      gtk_widget_set_visible (panel->arrangement_group, TRUE);
+      gtk_widget_set_visible (panel->display_multiple_displays, FALSE);
+      gtk_widget_set_visible (panel->arrangement_row, TRUE);
 
       /* Mirror is also invalid as it cannot be configured using this UI. */
       if (type == CC_DISPLAY_CONFIG_CLONE || type > CC_DISPLAY_CONFIG_LAST_VALID)
@@ -848,13 +827,15 @@ rebuild_ui (CcDisplayPanel *panel)
       type = CC_DISPLAY_CONFIG_JOIN;
 
       gtk_widget_set_visible (panel->display_settings_group, FALSE);
-      gtk_widget_set_visible (panel->config_type_switcher_row, FALSE);
-      gtk_widget_set_visible (panel->arrangement_group, FALSE);
+      gtk_widget_set_visible (panel->display_multiple_displays, FALSE);
+      gtk_widget_set_visible (panel->arrangement_row, FALSE);
 
       move_display_settings_to_main_page (panel);
     }
 
-  cc_display_settings_set_multimonitor (panel->settings, n_outputs > 1);
+  cc_display_settings_set_multimonitor (panel->settings,
+                                        n_outputs > 1 &&
+                                        type != CC_DISPLAY_CONFIG_CLONE);
 
   cc_panel_set_selected_type (panel, type);
 
@@ -1084,7 +1065,7 @@ cc_display_panel_init (CcDisplayPanel *self)
   gtk_widget_init_template (GTK_WIDGET (self));
 
   self->arrangement = cc_display_arrangement_new (NULL);
-  gtk_widget_set_size_request (GTK_WIDGET (self->arrangement), 400, 175);
+  gtk_widget_set_size_request (GTK_WIDGET (self->arrangement), 375, 175);
   adw_bin_set_child (self->arrangement_bin, GTK_WIDGET (self->arrangement));
 
   g_signal_connect_object (self->arrangement, "updated",
