@@ -26,21 +26,29 @@
 #include <math.h>
 
 #include "cc-night-light-page.h"
-#include "list-box-helper.h"
 
 #include "shell/cc-object-storage.h"
+#include "cc-display-config-manager-dbus.h"
 
 struct _CcNightLightPage {
-  GtkBin               parent;
+  AdwBin               parent;
 
+  GtkWidget           *night_light_settings;
   GtkWidget           *box_manual;
+  GtkButton           *button_from_am;
+  GtkButton           *button_from_pm;
+  GtkButton           *button_to_am;
+  GtkButton           *button_to_pm;
+  GtkWidget           *infobar_unsupported;
   GtkWidget           *infobar_disabled;
   GtkListBox          *listbox;
   GtkWidget           *scale_color_temperature;
   GtkWidget           *night_light_toggle_switch;
   GtkComboBox         *schedule_type_combo;
+  GtkWidget           *from_spinbuttons_box;
   GtkWidget           *spinbutton_from_hours;
   GtkWidget           *spinbutton_from_minutes;
+  GtkWidget           *to_spinbuttons_box;
   GtkWidget           *spinbutton_to_hours;
   GtkWidget           *spinbutton_to_minutes;
   GtkStack            *stack_from;
@@ -60,9 +68,11 @@ struct _CcNightLightPage {
   gboolean             ignore_value_changed;
   guint                timer_id;
   GDesktopClockFormat  clock_format;
+
+  CcDisplayConfigManager *config_manager;
 };
 
-G_DEFINE_TYPE (CcNightLightPage, cc_night_light_page, GTK_TYPE_BIN);
+G_DEFINE_TYPE (CcNightLightPage, cc_night_light_page, ADW_TYPE_BIN);
 
 #define CLOCK_SCHEMA     "org.gnome.desktop.interface"
 #define DISPLAY_SCHEMA   "org.gnome.settings-daemon.plugins.color"
@@ -74,7 +84,9 @@ dialog_adjustments_set_frac_hours (CcNightLightPage *self,
                                    gdouble           value,
                                    GtkAdjustment    *adj_hours,
                                    GtkAdjustment    *adj_mins,
-                                   GtkStack         *stack)
+                                   GtkStack         *stack,
+                                   GtkButton        *button_am,
+                                   GtkButton        *button_pm)
 {
   gdouble hours;
   gdouble mins = 0.f;
@@ -109,93 +121,104 @@ dialog_adjustments_set_frac_hours (CcNightLightPage *self,
   gtk_adjustment_set_value (GTK_ADJUSTMENT (adj_mins), mins);
   self->ignore_value_changed = FALSE;
 
-  if (is_24h)
-    gtk_stack_set_visible_child_name (stack, "blank");
-  else
-    gtk_stack_set_visible_child_name (stack, is_pm ? "pm" : "am");
+  gtk_widget_set_visible (GTK_WIDGET (stack), !is_24h);
+  gtk_stack_set_visible_child (stack, is_pm ? GTK_WIDGET (button_pm) : GTK_WIDGET (button_am));
 }
 
 static void
 dialog_update_state (CcNightLightPage *self)
 {
-  gboolean automatic;
-  gboolean disabled_until_tomorrow = FALSE;
-  gboolean enabled;
-  gdouble value = 0.f;
-
-  /* only show the infobar if we are disabled */
-  if (self->proxy_color != NULL)
+  if (cc_display_config_manager_get_night_light_supported (self->config_manager))
     {
-      g_autoptr(GVariant) disabled = NULL;
-      disabled = g_dbus_proxy_get_cached_property (self->proxy_color,
-                                                   "DisabledUntilTomorrow");
-      if (disabled != NULL)
-        disabled_until_tomorrow = g_variant_get_boolean (disabled);
-    }
-  gtk_widget_set_visible (self->infobar_disabled, disabled_until_tomorrow);
+      gboolean automatic;
+      gboolean disabled_until_tomorrow = FALSE;
+      gboolean enabled;
+      gdouble value = 0.f;
 
-  /* make things insensitive if the switch is disabled */
-  enabled = g_settings_get_boolean (self->settings_display, "night-light-enabled");
-  automatic = g_settings_get_boolean (self->settings_display, "night-light-schedule-automatic");
-
-  gtk_widget_set_sensitive (self->box_manual, enabled && !automatic);
-
-  gtk_combo_box_set_active_id (self->schedule_type_combo, automatic ? "automatic" : "manual");
-
-  /* set from */
-  if (automatic && self->proxy_color != NULL)
-    {
-      g_autoptr(GVariant) sunset = NULL;
-      sunset = g_dbus_proxy_get_cached_property (self->proxy_color, "Sunset");
-      if (sunset != NULL)
+      /* only show the infobar if we are disabled */
+      if (self->proxy_color != NULL)
         {
-          value = g_variant_get_double (sunset);
+          g_autoptr(GVariant) disabled = NULL;
+          disabled = g_dbus_proxy_get_cached_property (self->proxy_color,
+                                                       "DisabledUntilTomorrow");
+          if (disabled != NULL)
+            disabled_until_tomorrow = g_variant_get_boolean (disabled);
+        }
+      gtk_widget_set_visible (self->infobar_disabled, disabled_until_tomorrow);
+
+      /* make things insensitive if the switch is disabled */
+      enabled = g_settings_get_boolean (self->settings_display, "night-light-enabled");
+      automatic = g_settings_get_boolean (self->settings_display, "night-light-schedule-automatic");
+
+      gtk_widget_set_sensitive (self->box_manual, enabled && !automatic);
+
+      gtk_combo_box_set_active_id (self->schedule_type_combo, automatic ? "automatic" : "manual");
+
+      /* set from */
+      if (automatic && self->proxy_color != NULL)
+        {
+          g_autoptr(GVariant) sunset = NULL;
+          sunset = g_dbus_proxy_get_cached_property (self->proxy_color, "Sunset");
+          if (sunset != NULL)
+            {
+              value = g_variant_get_double (sunset);
+            }
+          else
+            {
+              value = 16.0f;
+              g_warning ("no sunset data, using %02.2f", value);
+            }
         }
       else
         {
-          value = 16.0f;
-          g_warning ("no sunset data, using %02.2f", value);
+          value = g_settings_get_double (self->settings_display, "night-light-schedule-from");
+          value = fmod (value, 24.f);
         }
-    }
-  else
-    {
-      value = g_settings_get_double (self->settings_display, "night-light-schedule-from");
-      value = fmod (value, 24.f);
-    }
-  dialog_adjustments_set_frac_hours (self, value,
-                                     self->adjustment_from_hours,
-                                     self->adjustment_from_minutes,
-                                     self->stack_from);
+      dialog_adjustments_set_frac_hours (self, value,
+                                         self->adjustment_from_hours,
+                                         self->adjustment_from_minutes,
+                                         self->stack_from,
+                                         self->button_from_am,
+                                         self->button_from_pm);
 
-  /* set to */
-  if (automatic && self->proxy_color != NULL)
-    {
-      g_autoptr(GVariant) sunset = NULL;
-      sunset = g_dbus_proxy_get_cached_property (self->proxy_color, "Sunrise");
-      if (sunset != NULL)
+      /* set to */
+      if (automatic && self->proxy_color != NULL)
         {
-          value = g_variant_get_double (sunset);
+          g_autoptr(GVariant) sunset = NULL;
+          sunset = g_dbus_proxy_get_cached_property (self->proxy_color, "Sunrise");
+          if (sunset != NULL)
+            {
+              value = g_variant_get_double (sunset);
+            }
+          else
+            {
+              value = 8.0f;
+              g_warning ("no sunrise data, using %02.2f", value);
+            }
         }
       else
         {
-          value = 8.0f;
-          g_warning ("no sunrise data, using %02.2f", value);
+          value = g_settings_get_double (self->settings_display, "night-light-schedule-to");
+          value = fmod (value, 24.f);
         }
+      dialog_adjustments_set_frac_hours (self, value,
+                                         self->adjustment_to_hours,
+                                         self->adjustment_to_minutes,
+                                         self->stack_to,
+                                         self->button_to_am,
+                                         self->button_to_pm);
+
+      self->ignore_value_changed = TRUE;
+      value = (gdouble) g_settings_get_uint (self->settings_display, "night-light-temperature");
+      gtk_adjustment_set_value (self->adjustment_color_temperature, value);
+      self->ignore_value_changed = FALSE;
     }
   else
     {
-      value = g_settings_get_double (self->settings_display, "night-light-schedule-to");
-      value = fmod (value, 24.f);
+      gtk_widget_set_visible (self->infobar_unsupported, TRUE);
+      gtk_widget_set_visible (self->infobar_disabled, FALSE);
+      gtk_widget_set_sensitive (self->night_light_settings, FALSE);
     }
-  dialog_adjustments_set_frac_hours (self, value,
-                                     self->adjustment_to_hours,
-                                     self->adjustment_to_minutes,
-                                     self->stack_to);
-
-  self->ignore_value_changed = TRUE;
-  value = (gdouble) g_settings_get_uint (self->settings_display, "night-light-temperature");
-  gtk_adjustment_set_value (self->adjustment_color_temperature, value);
-  self->ignore_value_changed = FALSE;
 }
 
 static void
@@ -372,10 +395,7 @@ dialog_color_temperature_value_changed_cb (GtkAdjustment    *adjustment,
 }
 
 static void
-dialog_color_properties_changed_cb (GDBusProxy       *proxy,
-                                    GVariant         *changed_properties,
-                                    GStrv             invalidated_properties,
-                                    CcNightLightPage *self)
+dialog_color_properties_changed_cb (CcNightLightPage *self)
 {
   dialog_update_state (self);
 }
@@ -400,7 +420,7 @@ dialog_got_proxy_cb (GObject      *source_object,
   self->proxy_color = proxy;
 
   g_signal_connect_object (self->proxy_color, "g-properties-changed",
-                           G_CALLBACK (dialog_color_properties_changed_cb), self, 0);
+                           G_CALLBACK (dialog_color_properties_changed_cb), self, G_CONNECT_SWAPPED);
   dialog_update_state (self);
   self->timer_id = g_timeout_add_seconds (10, dialog_tick_cb, self);
 }
@@ -433,7 +453,7 @@ dialog_format_minutes_combobox (GtkSpinButton    *spin,
   g_autofree gchar *text = NULL;
   adjustment = gtk_spin_button_get_adjustment (spin);
   text = g_strdup_printf ("%02.0f", gtk_adjustment_get_value (adjustment));
-  gtk_entry_set_text (GTK_ENTRY (spin), text);
+  gtk_editable_set_text (GTK_EDITABLE (spin), text);
   return TRUE;
 }
 
@@ -448,7 +468,7 @@ dialog_format_hours_combobox (GtkSpinButton      *spin,
     text = g_strdup_printf ("%.0f", gtk_adjustment_get_value (adjustment));
   else
     text = g_strdup_printf ("%02.0f", gtk_adjustment_get_value (adjustment));
-  gtk_entry_set_text (GTK_ENTRY (spin), text);
+  gtk_editable_set_text (GTK_EDITABLE (spin), text);
   return TRUE;
 }
 
@@ -464,7 +484,7 @@ dialog_update_adjustments (CcNightLightPage *self)
   else
     {
       if (gtk_adjustment_get_value (self->adjustment_from_hours) > 12)
-          gtk_stack_set_visible_child_name (self->stack_from, "pm");
+          gtk_stack_set_visible_child (self->stack_from, GTK_WIDGET (self->button_from_pm));
 
       gtk_adjustment_set_lower (self->adjustment_from_hours, 1);
       gtk_adjustment_set_upper (self->adjustment_from_hours, 12);
@@ -479,7 +499,7 @@ dialog_update_adjustments (CcNightLightPage *self)
   else
     {
       if (gtk_adjustment_get_value (self->adjustment_to_hours) > 12)
-          gtk_stack_set_visible_child_name (self->stack_to, "pm");
+          gtk_stack_set_visible_child (self->stack_to, GTK_WIDGET (self->button_to_pm));
 
       gtk_adjustment_set_lower (self->adjustment_to_hours, 1);
       gtk_adjustment_set_upper (self->adjustment_to_hours, 12);
@@ -487,19 +507,15 @@ dialog_update_adjustments (CcNightLightPage *self)
 }
 
 static void
-dialog_settings_changed_cb (GSettings        *settings_display,
-                            gchar            *key,
-                            CcNightLightPage *self)
+dialog_settings_changed_cb (CcNightLightPage *self)
 {
   dialog_update_state (self);
 }
 
 static void
-dialog_clock_settings_changed_cb (GSettings        *settings_display,
-                                  gchar            *key,
-                                  CcNightLightPage *self)
+dialog_clock_settings_changed_cb (CcNightLightPage *self)
 {
-  self->clock_format = g_settings_get_enum (settings_display, CLOCK_FORMAT_KEY);
+  self->clock_format = g_settings_get_enum (self->settings_clock, CLOCK_FORMAT_KEY);
 
   /* uncontionally widen this to avoid truncation */
   gtk_adjustment_set_lower (self->adjustment_from_hours, 0);
@@ -548,6 +564,13 @@ dialog_am_pm_to_button_clicked_cb (GtkButton        *button,
   g_debug ("new value = %.3f", value);
 }
 
+static void
+config_manager_changed_cb (CcDisplayConfigManager *config_manager,
+                           CcNightLightPage       *self)
+{
+  dialog_update_state (self);
+}
+
 /* GObject overrides */
 static void
 cc_night_light_page_finalize (GObject *object)
@@ -582,14 +605,22 @@ cc_night_light_page_class_init (CcNightLightPageClass *klass)
   gtk_widget_class_bind_template_child (widget_class, CcNightLightPage, adjustment_to_hours);
   gtk_widget_class_bind_template_child (widget_class, CcNightLightPage, adjustment_to_minutes);
   gtk_widget_class_bind_template_child (widget_class, CcNightLightPage, adjustment_color_temperature);
+  gtk_widget_class_bind_template_child (widget_class, CcNightLightPage, night_light_settings);
   gtk_widget_class_bind_template_child (widget_class, CcNightLightPage, box_manual);
+  gtk_widget_class_bind_template_child (widget_class, CcNightLightPage, button_from_am);
+  gtk_widget_class_bind_template_child (widget_class, CcNightLightPage, button_from_pm);
+  gtk_widget_class_bind_template_child (widget_class, CcNightLightPage, button_to_am);
+  gtk_widget_class_bind_template_child (widget_class, CcNightLightPage, button_to_pm);
+  gtk_widget_class_bind_template_child (widget_class, CcNightLightPage, infobar_unsupported);
   gtk_widget_class_bind_template_child (widget_class, CcNightLightPage, infobar_disabled);
   gtk_widget_class_bind_template_child (widget_class, CcNightLightPage, listbox);
   gtk_widget_class_bind_template_child (widget_class, CcNightLightPage, night_light_toggle_switch);
   gtk_widget_class_bind_template_child (widget_class, CcNightLightPage, schedule_type_combo);
   gtk_widget_class_bind_template_child (widget_class, CcNightLightPage, scale_color_temperature);
+  gtk_widget_class_bind_template_child (widget_class, CcNightLightPage, from_spinbuttons_box);
   gtk_widget_class_bind_template_child (widget_class, CcNightLightPage, spinbutton_from_hours);
   gtk_widget_class_bind_template_child (widget_class, CcNightLightPage, spinbutton_from_minutes);
+  gtk_widget_class_bind_template_child (widget_class, CcNightLightPage, to_spinbuttons_box);
   gtk_widget_class_bind_template_child (widget_class, CcNightLightPage, spinbutton_to_hours);
   gtk_widget_class_bind_template_child (widget_class, CcNightLightPage, spinbutton_to_minutes);
   gtk_widget_class_bind_template_child (widget_class, CcNightLightPage, stack_from);
@@ -615,11 +646,9 @@ cc_night_light_page_init (CcNightLightPage *self)
 
   gtk_widget_init_template (GTK_WIDGET (self));
 
-  gtk_list_box_set_header_func (self->listbox, cc_list_box_update_header_func, NULL, NULL);
-
   gtk_scale_add_mark (GTK_SCALE (self->scale_color_temperature),
                       1700, GTK_POS_BOTTOM,
-                      _("More Warm"));
+                      NULL);
 
   gtk_scale_add_mark (GTK_SCALE (self->scale_color_temperature),
                       2700, GTK_POS_BOTTOM,
@@ -631,12 +660,12 @@ cc_night_light_page_init (CcNightLightPage *self)
 
   gtk_scale_add_mark (GTK_SCALE (self->scale_color_temperature),
                       4700, GTK_POS_BOTTOM,
-                      _("Less Warm"));
+                      NULL);
 
   self->cancellable = g_cancellable_new ();
   self->settings_display = g_settings_new (DISPLAY_SCHEMA);
 
-  g_signal_connect (self->settings_display, "changed", G_CALLBACK (dialog_settings_changed_cb), self);
+  g_signal_connect_object (self->settings_display, "changed", G_CALLBACK (dialog_settings_changed_cb), self, G_CONNECT_SWAPPED);
 
   build_schedule_combo_row (self);
 
@@ -664,9 +693,9 @@ cc_night_light_page_init (CcNightLightPage *self)
   /* use custom CSS */
   provider = gtk_css_provider_new ();
   gtk_css_provider_load_from_resource (provider, "/org/gnome/control-center/display/night-light.css");
-  gtk_style_context_add_provider_for_screen (gdk_screen_get_default (),
-                                             GTK_STYLE_PROVIDER (provider),
-                                             GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+  gtk_style_context_add_provider_for_display (gdk_display_get_default (),
+                                              GTK_STYLE_PROVIDER (provider),
+                                              GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
 
   cc_object_storage_create_dbus_proxy (G_BUS_TYPE_SESSION,
                                        G_DBUS_PROXY_FLAGS_NONE,
@@ -690,10 +719,20 @@ cc_night_light_page_init (CcNightLightPage *self)
   self->settings_clock = g_settings_new (CLOCK_SCHEMA);
   self->clock_format = g_settings_get_enum (self->settings_clock, CLOCK_FORMAT_KEY);
   dialog_update_adjustments (self);
-  g_signal_connect (self->settings_clock,
-                    "changed::" CLOCK_FORMAT_KEY,
-                    G_CALLBACK (dialog_clock_settings_changed_cb),
-                    self);
+  g_signal_connect_object (self->settings_clock,
+                           "changed::" CLOCK_FORMAT_KEY,
+                           G_CALLBACK (dialog_clock_settings_changed_cb),
+                           self, G_CONNECT_SWAPPED);
+
+  if (gtk_widget_get_default_direction () == GTK_TEXT_DIR_RTL)
+    {
+      gtk_widget_set_direction (self->to_spinbuttons_box, GTK_TEXT_DIR_LTR);
+      gtk_widget_set_direction (self->from_spinbuttons_box, GTK_TEXT_DIR_LTR);
+    }
+
+  self->config_manager = cc_display_config_manager_dbus_new ();
+  g_signal_connect (self->config_manager, "changed",
+                    G_CALLBACK (config_manager_changed_cb), self);
 
   dialog_update_state (self);
 }
