@@ -66,12 +66,10 @@ security_item_changed_cb (CEPage8021xSecurity *self)
 static void
 finish_setup (CEPage8021xSecurity *self, gpointer unused, GError *error, gpointer user_data)
 {
-	GtkWidget *parent;
-
 	if (error)
 		return;
 
-        self->group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
+	self->group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 
 	self->security = ws_wpa_eap_new (self->connection);
 	if (!self->security) {
@@ -79,19 +77,18 @@ finish_setup (CEPage8021xSecurity *self, gpointer unused, GError *error, gpointe
 		return;
 	}
 
-        g_signal_connect_object (WIRELESS_SECURITY (self->security), "changed", G_CALLBACK (security_item_changed_cb), self, G_CONNECT_SWAPPED);
-	parent = gtk_widget_get_parent (GTK_WIDGET (self->security));
-	if (parent)
-		gtk_container_remove (GTK_CONTAINER (parent), GTK_WIDGET (self->security));
+	g_signal_connect_object (WIRELESS_SECURITY (self->security), "changed", G_CALLBACK (security_item_changed_cb), self, G_CONNECT_SWAPPED);
+	if (gtk_widget_get_parent (GTK_WIDGET (self->security)))
+		gtk_box_remove (self->box, GTK_WIDGET (self->security));
 
 	gtk_switch_set_active (self->enable_8021x_switch, self->initial_have_8021x);
 	g_signal_connect_object (self->enable_8021x_switch, "notify::active", G_CALLBACK (enable_toggled), self, G_CONNECT_SWAPPED);
 	gtk_widget_set_sensitive (GTK_WIDGET (self->security), self->initial_have_8021x);
 
-        gtk_size_group_add_widget (self->group, GTK_WIDGET (self->security_label));
-        wireless_security_add_to_size_group (WIRELESS_SECURITY (self->security), self->group);
+	gtk_size_group_add_widget (self->group, GTK_WIDGET (self->security_label));
+	wireless_security_add_to_size_group (WIRELESS_SECURITY (self->security), self->group);
 
-	gtk_container_add (GTK_CONTAINER (self->box), GTK_WIDGET (self->security));
+	gtk_box_append (self->box, GTK_WIDGET (self->security));
 
 }
 
@@ -125,22 +122,18 @@ ce_page_8021x_security_validate (CEPage *cepage, NMConnection *connection, GErro
 		valid = wireless_security_validate (WIRELESS_SECURITY (self->security), error);
 		if (valid) {
 			g_autoptr(NMConnection) tmp_connection = NULL;
-			NMSetting *s_con;
 
 			/* Here's a nice hack to work around the fact that ws_802_1x_fill_connection needs wireless setting. */
-			tmp_connection = nm_simple_connection_new ();
+			tmp_connection = nm_simple_connection_new_clone (connection);
 			nm_connection_add_setting (tmp_connection, nm_setting_wireless_new ());
 
-			/* temp connection needs a 'connection' setting too, since most of
-			 * the EAP methods need the UUID for CA cert ignore stuff.
-			 */
-			s_con = nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION);
-			nm_connection_add_setting (tmp_connection, nm_setting_duplicate (s_con));
-
 			ws_wpa_eap_fill_connection (self->security, tmp_connection);
 
+			/* NOTE: It is important we create a copy of the settings, as the
+			 * secrets might be cleared otherwise.
+			 */
 			s_8021x = nm_connection_get_setting (tmp_connection, NM_TYPE_SETTING_802_1X);
-			nm_connection_add_setting (connection, NM_SETTING (g_object_ref (s_8021x)));
+			nm_connection_add_setting (connection, nm_setting_duplicate (NM_SETTING (s_8021x)));
 		}
 	} else {
 		nm_connection_remove_setting (connection, NM_TYPE_SETTING_802_1X);
@@ -162,7 +155,7 @@ ce_page_8021x_security_dispose (GObject *object)
 	CEPage8021xSecurity *self = CE_PAGE_8021X_SECURITY (object);
 
         g_clear_object (&self->connection);
-        g_clear_object (&self->security);
+        g_clear_pointer ((GtkWidget **) &self->security, gtk_widget_unparent);
         g_clear_object (&self->group);
 
 	G_OBJECT_CLASS (ce_page_8021x_security_parent_class)->dispose (object);
