@@ -27,7 +27,6 @@
 #include <glib/gi18n.h>
 #include <NetworkManager.h>
 
-#include "list-box-helper.h"
 #include "ce-ip-address-entry.h"
 #include "ce-page.h"
 #include "ce-page-ip6.h"
@@ -39,24 +38,32 @@ static void ensure_empty_routes_row (CEPageIP6 *self);
 
 struct _CEPageIP6
 {
-        GtkScrolledWindow parent;
+        AdwBin            parent;
 
         GtkBox            *address_box;
+        GtkLabel          *address_address_label;
+        GtkLabel          *address_prefix_label;
+        GtkLabel          *address_gateway_label;
         GtkSizeGroup      *address_sizegroup;
         GtkSwitch         *auto_dns_switch;
         GtkSwitch         *auto_routes_switch;
-        GtkRadioButton    *automatic_radio;
+        GtkCheckButton    *automatic_radio;
         GtkBox            *content_box;
-        GtkRadioButton    *dhcp_radio;
-        GtkRadioButton    *disabled_radio;
+        GtkCheckButton    *dhcp_radio;
+        GtkCheckButton    *disabled_radio;
         GtkEntry          *dns_entry;
-        GtkRadioButton    *local_radio;
-        GtkRadioButton    *manual_radio;
+        GtkCheckButton    *local_radio;
+        GtkGrid           *main_box;
+        GtkCheckButton    *manual_radio;
         GtkCheckButton    *never_default_check;
         GtkBox            *routes_box;
+        GtkLabel         *routes_address_label;
+        GtkLabel         *routes_prefix_label;
+        GtkLabel         *routes_gateway_label;
+        GtkLabel         *routes_metric_label;
         GtkSizeGroup      *routes_metric_sizegroup;
         GtkSizeGroup      *routes_sizegroup;
-        GtkRadioButton    *shared_radio;
+        GtkCheckButton    *shared_radio;
 
         NMSettingIPConfig *setting;
 
@@ -66,7 +73,7 @@ struct _CEPageIP6
 
 static void ce_page_iface_init (CEPageInterface *);
 
-G_DEFINE_TYPE_WITH_CODE (CEPageIP6, ce_page_ip6, GTK_TYPE_SCROLLED_WINDOW,
+G_DEFINE_TYPE_WITH_CODE (CEPageIP6, ce_page_ip6, ADW_TYPE_BIN,
                          G_IMPLEMENT_INTERFACE (ce_page_get_type (), ce_page_iface_init))
 
 enum {
@@ -90,15 +97,15 @@ method_changed (CEPageIP6 *self)
         gboolean dns_enabled;
         gboolean routes_enabled;
 
-        if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->disabled_radio)) ||
-            gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->shared_radio))) {
+        if (gtk_check_button_get_active (self->disabled_radio) ||
+            gtk_check_button_get_active (self->shared_radio)) {
                 addr_enabled = FALSE;
                 dns_enabled = FALSE;
                 routes_enabled = FALSE;
         } else {
-                addr_enabled = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->manual_radio));
-                dns_enabled = !gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->local_radio));
-                routes_enabled = !gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->local_radio));
+                addr_enabled = gtk_check_button_get_active (self->manual_radio);
+                dns_enabled = !gtk_check_button_get_active (self->local_radio);
+                routes_enabled = !gtk_check_button_get_active (self->local_radio);
         }
 
         gtk_widget_set_visible (GTK_WIDGET (self->address_box), addr_enabled);
@@ -112,23 +119,24 @@ method_changed (CEPageIP6 *self)
 static void
 update_row_sensitivity (CEPageIP6 *self, GtkWidget *list)
 {
-        g_autoptr(GList) children = NULL;
+        GtkWidget *child;
         gint rows = 0, i = 0;
 
-        children = gtk_container_get_children (GTK_CONTAINER (list));
-        for (GList *l = children; l; l = l->next) {
-                GtkWidget *row = l->data;
+        for (child = gtk_widget_get_first_child (GTK_WIDGET (list));
+             child;
+             child = gtk_widget_get_next_sibling (child)) {
                 GtkWidget *button;
 
-                button = GTK_WIDGET (g_object_get_data (G_OBJECT (row), "delete-button"));
+                button = GTK_WIDGET (g_object_get_data (G_OBJECT (child), "delete-button"));
                 if (button != NULL)
                         rows++;
         }
-        for (GList *l = children; l; l = l->next) {
-                GtkWidget *row = l->data;
+        for (child = gtk_widget_get_first_child (GTK_WIDGET (list));
+             child;
+             child = gtk_widget_get_next_sibling (child)) {
                 GtkWidget *button;
 
-                button = GTK_WIDGET (g_object_get_data (G_OBJECT (row), "delete-button"));
+                button = GTK_WIDGET (g_object_get_data (G_OBJECT (child), "delete-button"));
                 if (button != NULL)
                         gtk_widget_set_sensitive (button, rows > 1 && ++i < rows);
         }
@@ -145,7 +153,7 @@ remove_row (CEPageIP6 *self, GtkButton *button)
         row = gtk_widget_get_parent (row_box);
         list = gtk_widget_get_parent (row);
 
-        gtk_container_remove (GTK_CONTAINER (list), row);
+        gtk_list_box_remove (GTK_LIST_BOX (list), row);
 
         ce_page_changed (CE_PAGE (self));
 
@@ -155,19 +163,20 @@ remove_row (CEPageIP6 *self, GtkButton *button)
 static gboolean
 validate_row (GtkWidget *row)
 {
+        GtkWidget *child;
         GtkWidget *box;
-        g_autoptr(GList) children = NULL;
         gboolean valid;
 
         valid = FALSE;
-        box = gtk_bin_get_child (GTK_BIN (row));
-        children = gtk_container_get_children (GTK_CONTAINER (box));
+        box = gtk_list_box_row_get_child (GTK_LIST_BOX_ROW (row));
 
-        for (GList *l = children; l != NULL; l = l->next) {
-                if (!GTK_IS_ENTRY (l->data))
+        for (child = gtk_widget_get_first_child (box);
+             child;
+             child = gtk_widget_get_next_sibling (child)) {
+                if (!GTK_IS_ENTRY (child))
                         continue;
 
-                valid = valid || gtk_entry_get_text_length (l->data) > 0;
+                valid = valid || gtk_entry_get_text_length (GTK_ENTRY (child)) > 0;
         }
 
         return valid;
@@ -183,55 +192,62 @@ add_address_row (CEPageIP6   *self,
         GtkWidget *row_box;
         GtkWidget *widget;
         GtkWidget *delete_button;
-        GtkWidget *image;
 
         row = gtk_list_box_row_new ();
+        gtk_list_box_row_set_activatable (GTK_LIST_BOX_ROW (row), FALSE);
 
         row_box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
-        gtk_style_context_add_class (gtk_widget_get_style_context (row_box), "linked");
+        gtk_widget_add_css_class (row_box, "linked");
 
         widget = GTK_WIDGET (ce_ip_address_entry_new (AF_INET6));
         g_signal_connect_object (widget, "changed", G_CALLBACK (ce_page_changed), self, G_CONNECT_SWAPPED);
         g_signal_connect_object (widget, "activate", G_CALLBACK (ensure_empty_address_row), self, G_CONNECT_SWAPPED);
         g_object_set_data (G_OBJECT (row), "address", widget);
-        gtk_entry_set_text (GTK_ENTRY (widget), address);
-        gtk_entry_set_width_chars (GTK_ENTRY (widget), 16);
+        gtk_editable_set_text (GTK_EDITABLE (widget), address);
+        gtk_editable_set_width_chars (GTK_EDITABLE (widget), 16);
         gtk_widget_set_hexpand (widget, TRUE);
-        gtk_container_add (GTK_CONTAINER (row_box), widget);
+        gtk_accessible_update_relation (GTK_ACCESSIBLE (widget),
+                                        GTK_ACCESSIBLE_RELATION_LABELLED_BY, self->address_address_label, NULL,
+                                        -1);
+        gtk_box_append (GTK_BOX (row_box), widget);
 
         widget = gtk_entry_new ();
         g_signal_connect_object (widget, "changed", G_CALLBACK (ce_page_changed), self, G_CONNECT_SWAPPED);
         g_signal_connect_object (widget, "activate", G_CALLBACK (ensure_empty_address_row), self, G_CONNECT_SWAPPED);
         g_object_set_data (G_OBJECT (row), "prefix", widget);
-        gtk_entry_set_text (GTK_ENTRY (widget), network);
-        gtk_entry_set_width_chars (GTK_ENTRY (widget), 16);
+        gtk_editable_set_text (GTK_EDITABLE (widget), network);
+        gtk_editable_set_width_chars (GTK_EDITABLE (widget), 16);
         gtk_widget_set_hexpand (widget, TRUE);
-        gtk_container_add (GTK_CONTAINER (row_box), widget);
+        gtk_accessible_update_relation (GTK_ACCESSIBLE (widget),
+                                        GTK_ACCESSIBLE_RELATION_LABELLED_BY, self->address_prefix_label, NULL,
+                                        -1);
+        gtk_box_append (GTK_BOX (row_box), widget);
 
         widget = GTK_WIDGET (ce_ip_address_entry_new (AF_INET6));
         g_signal_connect_object (widget, "changed", G_CALLBACK (ce_page_changed), self, G_CONNECT_SWAPPED);
         g_signal_connect_object (widget, "activate", G_CALLBACK (ensure_empty_address_row), self, G_CONNECT_SWAPPED);
         g_object_set_data (G_OBJECT (row), "gateway", widget);
-        gtk_entry_set_text (GTK_ENTRY (widget), gateway ? gateway : "");
-        gtk_entry_set_width_chars (GTK_ENTRY (widget), 16);
+        gtk_editable_set_text (GTK_EDITABLE (widget), gateway ? gateway : "");
+        gtk_editable_set_width_chars (GTK_EDITABLE (widget), 16);
         gtk_widget_set_hexpand (widget, TRUE);
-        gtk_container_add (GTK_CONTAINER (row_box), widget);
+        gtk_accessible_update_relation (GTK_ACCESSIBLE (widget),
+                                        GTK_ACCESSIBLE_RELATION_LABELLED_BY, self->address_gateway_label, NULL,
+                                        -1);
+        gtk_box_append (GTK_BOX (row_box), widget);
 
-        delete_button = gtk_button_new ();
+        delete_button = gtk_button_new_from_icon_name ("edit-delete-symbolic");
         gtk_widget_set_sensitive (delete_button, FALSE);
-        gtk_style_context_add_class (gtk_widget_get_style_context (delete_button), "image-button");
         g_signal_connect_object (delete_button, "clicked", G_CALLBACK (remove_row), self, G_CONNECT_SWAPPED);
-        image = gtk_image_new_from_icon_name ("edit-delete-symbolic", GTK_ICON_SIZE_MENU);
-        atk_object_set_name (gtk_widget_get_accessible (delete_button), _("Delete Address"));
-        gtk_button_set_image (GTK_BUTTON (delete_button), image);
-        gtk_container_add (GTK_CONTAINER (row_box), delete_button);
+        gtk_accessible_update_property (GTK_ACCESSIBLE (delete_button),
+                                        GTK_ACCESSIBLE_PROPERTY_LABEL, _("Delete Address"),
+                                        -1);
+        gtk_box_append (GTK_BOX (row_box), delete_button);
         g_object_set_data (G_OBJECT (row), "delete-button", delete_button);
 
         gtk_size_group_add_widget (self->address_sizegroup, delete_button);
 
-        gtk_container_add (GTK_CONTAINER (row), row_box);
-        gtk_widget_show_all (row);
-        gtk_container_add (GTK_CONTAINER (self->address_list), row);
+        gtk_list_box_row_set_child (GTK_LIST_BOX_ROW (row), row_box);
+        gtk_list_box_append (GTK_LIST_BOX (self->address_list), row);
 
         update_row_sensitivity (self, self->address_list);
 }
@@ -239,14 +255,10 @@ add_address_row (CEPageIP6   *self,
 static void
 ensure_empty_address_row (CEPageIP6 *self)
 {
-        g_autoptr(GList) children = NULL;
-        GList *l;
-
-        children = gtk_container_get_children (GTK_CONTAINER (self->address_list));
-        l = g_list_last (children);
+        GtkWidget *child = gtk_widget_get_last_child (self->address_list);
 
         /* Add the last, stub row if needed*/
-        if (!l || validate_row (l->data))
+        if (!child || validate_row (child))
                 add_address_row (self, "", "", "");
 }
 
@@ -258,8 +270,7 @@ add_address_box (CEPageIP6 *self)
 
         self->address_list = list = gtk_list_box_new ();
         gtk_list_box_set_selection_mode (GTK_LIST_BOX (list), GTK_SELECTION_NONE);
-        gtk_list_box_set_header_func (GTK_LIST_BOX (list), cc_list_box_update_header_func, NULL, NULL);
-        gtk_container_add (GTK_CONTAINER (self->address_box), list);
+        gtk_box_append (self->address_box, list);
 
         for (i = 0; i < nm_setting_ip_config_get_num_addresses (self->setting); i++) {
                 NMIPAddress *addr;
@@ -272,8 +283,6 @@ add_address_box (CEPageIP6 *self)
         }
         if (nm_setting_ip_config_get_num_addresses (self->setting) == 0)
                 ensure_empty_address_row (self);
-
-        gtk_widget_show_all (GTK_WIDGET (self->address_box));
 }
 
 static void
@@ -299,7 +308,7 @@ add_dns_section (CEPageIP6 *self)
 
         }
 
-        gtk_entry_set_text (self->dns_entry, string->str);
+        gtk_editable_set_text (GTK_EDITABLE (self->dns_entry), string->str);
 
         g_signal_connect_object (self->dns_entry, "notify::text", G_CALLBACK (ce_page_changed), self, G_CONNECT_SWAPPED);
 
@@ -317,67 +326,77 @@ add_route_row (CEPageIP6   *self,
         GtkWidget *row_box;
         GtkWidget *widget;
         GtkWidget *delete_button;
-        GtkWidget *image;
 
         row = gtk_list_box_row_new ();
+        gtk_list_box_row_set_activatable (GTK_LIST_BOX_ROW (row), FALSE);
 
         row_box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
-        gtk_style_context_add_class (gtk_widget_get_style_context (row_box), "linked");
+        gtk_widget_add_css_class (row_box, "linked");
 
         widget = GTK_WIDGET (ce_ip_address_entry_new (AF_INET6));
         g_signal_connect_object (widget, "changed", G_CALLBACK (ce_page_changed), self, G_CONNECT_SWAPPED);
         g_signal_connect_object (widget, "activate", G_CALLBACK (ensure_empty_routes_row), self, G_CONNECT_SWAPPED);
         g_object_set_data (G_OBJECT (row), "address", widget);
-        gtk_entry_set_text (GTK_ENTRY (widget), address);
-        gtk_entry_set_width_chars (GTK_ENTRY (widget), 16);
+        gtk_editable_set_text (GTK_EDITABLE (widget), address);
+        gtk_editable_set_width_chars (GTK_EDITABLE (widget), 16);
         gtk_widget_set_hexpand (widget, TRUE);
-        gtk_container_add (GTK_CONTAINER (row_box), widget);
+        gtk_accessible_update_relation (GTK_ACCESSIBLE (widget),
+                                        GTK_ACCESSIBLE_RELATION_LABELLED_BY, self->routes_address_label, NULL,
+                                        -1);
+        gtk_box_append (GTK_BOX (row_box), widget);
 
         widget = gtk_entry_new ();
         g_signal_connect_object (widget, "changed", G_CALLBACK (ce_page_changed), self, G_CONNECT_SWAPPED);
         g_signal_connect_object (widget, "activate", G_CALLBACK (ensure_empty_routes_row), self, G_CONNECT_SWAPPED);
         g_object_set_data (G_OBJECT (row), "prefix", widget);
-        gtk_entry_set_text (GTK_ENTRY (widget), prefix ? prefix : "");
-        gtk_entry_set_width_chars (GTK_ENTRY (widget), 16);
+        gtk_editable_set_text (GTK_EDITABLE (widget), prefix ? prefix : "");
+        gtk_editable_set_width_chars (GTK_EDITABLE (widget), 16);
         gtk_widget_set_hexpand (widget, TRUE);
-        gtk_container_add (GTK_CONTAINER (row_box), widget);
+        gtk_accessible_update_relation (GTK_ACCESSIBLE (widget),
+                                        GTK_ACCESSIBLE_RELATION_LABELLED_BY, self->routes_prefix_label, NULL,
+                                        -1);
+        gtk_box_append (GTK_BOX (row_box), widget);
 
         widget = GTK_WIDGET (ce_ip_address_entry_new (AF_INET6));
         g_signal_connect_object (widget, "changed", G_CALLBACK (ce_page_changed), self, G_CONNECT_SWAPPED);
         g_signal_connect_object (widget, "activate", G_CALLBACK (ensure_empty_routes_row), self, G_CONNECT_SWAPPED);
         g_object_set_data (G_OBJECT (row), "gateway", widget);
-        gtk_entry_set_text (GTK_ENTRY (widget), gateway);
-        gtk_entry_set_width_chars (GTK_ENTRY (widget), 16);
+        gtk_editable_set_text (GTK_EDITABLE (widget), gateway);
+        gtk_editable_set_width_chars (GTK_EDITABLE (widget), 16);
         gtk_widget_set_hexpand (widget, TRUE);
-        gtk_container_add (GTK_CONTAINER (row_box), widget);
+        gtk_accessible_update_relation (GTK_ACCESSIBLE (widget),
+                                        GTK_ACCESSIBLE_RELATION_LABELLED_BY, self->routes_gateway_label, NULL,
+                                        -1);
+        gtk_box_append (GTK_BOX (row_box), widget);
 
         widget = gtk_entry_new ();
         g_signal_connect_object (widget, "changed", G_CALLBACK (ce_page_changed), self, G_CONNECT_SWAPPED);
         g_signal_connect_object (widget, "activate", G_CALLBACK (ensure_empty_routes_row), self, G_CONNECT_SWAPPED);
         g_object_set_data (G_OBJECT (row), "metric", widget);
-        gtk_entry_set_text (GTK_ENTRY (widget), metric ? metric : "");
-        gtk_entry_set_width_chars (GTK_ENTRY (widget), 5);
+        gtk_editable_set_text (GTK_EDITABLE (widget), metric ? metric : "");
+        gtk_editable_set_width_chars (GTK_EDITABLE (widget), 5);
         gtk_widget_set_hexpand (widget, TRUE);
-        gtk_container_add (GTK_CONTAINER (row_box), widget);
+        gtk_accessible_update_relation (GTK_ACCESSIBLE (widget),
+                                        GTK_ACCESSIBLE_RELATION_LABELLED_BY, self->routes_prefix_label, NULL,
+                                        -1);
+        gtk_box_append (GTK_BOX (row_box), widget);
 
         gtk_size_group_add_widget (self->routes_metric_sizegroup, widget);
 
-        delete_button = gtk_button_new ();
-        gtk_style_context_add_class (gtk_widget_get_style_context (delete_button), "image-button");
+        delete_button = gtk_button_new_from_icon_name ("edit-delete-symbolic");
         g_signal_connect_object (delete_button, "clicked", G_CALLBACK (remove_row), self, G_CONNECT_SWAPPED);
-        image = gtk_image_new_from_icon_name ("edit-delete-symbolic", GTK_ICON_SIZE_MENU);
-        atk_object_set_name (gtk_widget_get_accessible (delete_button), _("Delete Route"));
-        gtk_button_set_image (GTK_BUTTON (delete_button), image);
+        gtk_accessible_update_property (GTK_ACCESSIBLE (delete_button),
+                                        GTK_ACCESSIBLE_PROPERTY_LABEL, _("Delete Route"),
+                                        -1);
         gtk_widget_set_halign (delete_button, GTK_ALIGN_CENTER);
         gtk_widget_set_valign (delete_button, GTK_ALIGN_CENTER);
-        gtk_container_add (GTK_CONTAINER (row_box), delete_button);
+        gtk_box_append (GTK_BOX (row_box), delete_button);
         g_object_set_data (G_OBJECT (row), "delete-button", delete_button);
 
         gtk_size_group_add_widget (self->routes_sizegroup, delete_button);
 
-        gtk_container_add (GTK_CONTAINER (row), row_box);
-        gtk_widget_show_all (row);
-        gtk_container_add (GTK_CONTAINER (self->routes_list), row);
+        gtk_list_box_row_set_child (GTK_LIST_BOX_ROW (row), row_box);
+        gtk_list_box_append (GTK_LIST_BOX (self->routes_list), row);
 
         update_row_sensitivity (self, self->routes_list);
 }
@@ -385,17 +404,10 @@ add_route_row (CEPageIP6   *self,
 static void
 ensure_empty_routes_row (CEPageIP6 *self)
 {
-        g_autoptr(GList) children = NULL;
-        GList *l;
-
-        children = gtk_container_get_children (GTK_CONTAINER (self->routes_list));
-        l = g_list_last (children);
-
-        while (l && l->next)
-                l = l->next;
+        GtkWidget *child = gtk_widget_get_last_child (self->routes_list);
 
         /* Add the last, stub row if needed*/
-        if (!l || validate_row (l->data))
+        if (!child || validate_row (child))
                 add_route_row (self, "", NULL, "", NULL);
 }
 
@@ -413,8 +425,7 @@ add_routes_box (CEPageIP6 *self)
 
         self->routes_list = list = gtk_list_box_new ();
         gtk_list_box_set_selection_mode (GTK_LIST_BOX (list), GTK_SELECTION_NONE);
-        gtk_list_box_set_header_func (GTK_LIST_BOX (list), cc_list_box_update_header_func, NULL, NULL);
-        gtk_container_add (GTK_CONTAINER (self->routes_box), list);
+        gtk_box_append (self->routes_box, list);
         gtk_switch_set_active (self->auto_routes_switch, !nm_setting_ip_config_get_ignore_auto_routes (self->setting));
         g_signal_connect_object (self->auto_routes_switch, "notify::active", G_CALLBACK (ce_page_changed), self, G_CONNECT_SWAPPED);
 
@@ -433,8 +444,6 @@ add_routes_box (CEPageIP6 *self)
         }
         if (nm_setting_ip_config_get_num_routes (self->setting) == 0)
                 add_empty_route_row (self);
-
-        gtk_widget_show_all (GTK_WIDGET (self->routes_box));
 }
 
 static void
@@ -472,8 +481,8 @@ connect_ip6_page (CEPageIP6 *self)
                 method = IP6_METHOD_DISABLED;
         }
 
-        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->never_default_check),
-                                      nm_setting_ip_config_get_never_default (self->setting));
+        gtk_check_button_set_active (GTK_CHECK_BUTTON (self->never_default_check),
+                                     nm_setting_ip_config_get_never_default (self->setting));
         g_signal_connect_object (self->never_default_check, "toggled", G_CALLBACK (ce_page_changed), self, G_CONNECT_SWAPPED);
 
         g_signal_connect_object (self->automatic_radio, "toggled", G_CALLBACK (method_changed), self, G_CONNECT_SWAPPED);
@@ -484,22 +493,22 @@ connect_ip6_page (CEPageIP6 *self)
 
         switch (method) {
         case IP6_METHOD_AUTO:
-                gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->automatic_radio), TRUE);
+                gtk_check_button_set_active (GTK_CHECK_BUTTON (self->automatic_radio), TRUE);
                 break;
         case IP6_METHOD_DHCP:
-                gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->dhcp_radio), TRUE);
+                gtk_check_button_set_active (GTK_CHECK_BUTTON (self->dhcp_radio), TRUE);
                 break;
         case IP6_METHOD_LINK_LOCAL:
-                gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->local_radio), TRUE);
+                gtk_check_button_set_active (GTK_CHECK_BUTTON (self->local_radio), TRUE);
                 break;
         case IP6_METHOD_MANUAL:
-                gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->manual_radio), TRUE);
+                gtk_check_button_set_active (GTK_CHECK_BUTTON (self->manual_radio), TRUE);
                 break;
         case IP6_METHOD_SHARED:
-                gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->shared_radio), TRUE);
+                gtk_check_button_set_active (GTK_CHECK_BUTTON (self->shared_radio), TRUE);
                 break;
         case IP6_METHOD_DISABLED:
-                gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->disabled_radio), TRUE);
+                gtk_check_button_set_active (GTK_CHECK_BUTTON (self->disabled_radio), TRUE);
                 break;
         default:
                 break;
@@ -511,41 +520,44 @@ connect_ip6_page (CEPageIP6 *self)
 static gboolean
 ui_to_setting (CEPageIP6 *self)
 {
+        GtkWidget *child;
         const gchar *method;
         gboolean ignore_auto_dns;
         gboolean ignore_auto_routes;
         gboolean never_default;
-        g_autoptr(GList) address_children = NULL;
-        g_autoptr(GList) routes_children = NULL;
+        gboolean add_addresses = FALSE;
+        gboolean add_routes = FALSE;
         gboolean ret = TRUE;
         GStrv dns_addresses = NULL;
         gchar *dns_text = NULL;
         guint i;
 
-        if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->disabled_radio)))
+        if (gtk_check_button_get_active (self->disabled_radio))
                 method = NM_SETTING_IP6_CONFIG_METHOD_DISABLED;
-        else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->manual_radio)))
+        else if (gtk_check_button_get_active (self->manual_radio))
                 method = NM_SETTING_IP6_CONFIG_METHOD_MANUAL;
-        else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->local_radio)))
+        else if (gtk_check_button_get_active (self->local_radio))
                 method = NM_SETTING_IP6_CONFIG_METHOD_LINK_LOCAL;
-        else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->dhcp_radio)))
+        else if (gtk_check_button_get_active (self->dhcp_radio))
                 method = NM_SETTING_IP6_CONFIG_METHOD_DHCP;
-        else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->automatic_radio)))
+        else if (gtk_check_button_get_active (self->automatic_radio))
                 method = NM_SETTING_IP6_CONFIG_METHOD_AUTO;
-        else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->shared_radio)))
+        else if (gtk_check_button_get_active (self->shared_radio))
                 method = NM_SETTING_IP6_CONFIG_METHOD_SHARED;
 
         nm_setting_ip_config_clear_addresses (self->setting);
         if (g_str_equal (method, NM_SETTING_IP6_CONFIG_METHOD_MANUAL)) {
-                address_children = gtk_container_get_children (GTK_CONTAINER (self->address_list));
+                add_addresses = TRUE;
         } else {
                 g_object_set (G_OBJECT (self->setting),
                               NM_SETTING_IP_CONFIG_GATEWAY, NULL,
                               NULL);
        }
 
-        for (GList *l = address_children; l; l = l->next) {
-                GtkWidget *row = l->data;
+        for (child = gtk_widget_get_first_child (self->address_list);
+             add_addresses && child;
+             child = gtk_widget_get_next_sibling (child)) {
+                GtkWidget *row = child;
                 CEIPAddressEntry *address_entry;
                 CEIPAddressEntry *gateway_entry;
                 const gchar *text_prefix;
@@ -557,7 +569,7 @@ ui_to_setting (CEPageIP6 *self)
                 if (!address_entry)
                         continue;
 
-                text_prefix = gtk_entry_get_text (GTK_ENTRY (g_object_get_data (G_OBJECT (row), "prefix")));
+                text_prefix = gtk_editable_get_text (GTK_EDITABLE (g_object_get_data (G_OBJECT (row), "prefix")));
                 gateway_entry = CE_IP_ADDRESS_ENTRY (g_object_get_data (G_OBJECT (row), "gateway"));
 
                 if (ce_ip_address_entry_is_empty (address_entry) && !*text_prefix && ce_ip_address_entry_is_empty (gateway_entry)) {
@@ -583,19 +595,19 @@ ui_to_setting (CEPageIP6 *self)
                 if (!ret)
                         continue;
 
-                addr = nm_ip_address_new (AF_INET6, gtk_entry_get_text (GTK_ENTRY (address_entry)), prefix, NULL);
+                addr = nm_ip_address_new (AF_INET6, gtk_editable_get_text (GTK_EDITABLE (address_entry)), prefix, NULL);
                 if (!ce_ip_address_entry_is_empty (gateway_entry))
                         g_object_set (G_OBJECT (self->setting),
-                                      NM_SETTING_IP_CONFIG_GATEWAY, gtk_entry_get_text (GTK_ENTRY (gateway_entry)),
+                                      NM_SETTING_IP_CONFIG_GATEWAY, gtk_editable_get_text (GTK_EDITABLE (gateway_entry)),
                                       NULL);
                 nm_setting_ip_config_add_address (self->setting, addr);
 
-                if (!l || !l->next)
+                if (!gtk_widget_get_next_sibling (row))
                         ensure_empty_address_row (self);
         }
 
         nm_setting_ip_config_clear_dns (self->setting);
-        dns_text = g_strstrip (g_strdup (gtk_entry_get_text (GTK_ENTRY (self->dns_entry))));
+        dns_text = g_strstrip (g_strdup (gtk_editable_get_text (GTK_EDITABLE (self->dns_entry))));
 
         if (g_str_equal (method, NM_SETTING_IP6_CONFIG_METHOD_AUTO) ||
             g_str_equal (method, NM_SETTING_IP6_CONFIG_METHOD_DHCP) ||
@@ -625,13 +637,14 @@ ui_to_setting (CEPageIP6 *self)
         }
 
         nm_setting_ip_config_clear_routes (self->setting);
-        if (g_str_equal (method, NM_SETTING_IP6_CONFIG_METHOD_AUTO) ||
-            g_str_equal (method, NM_SETTING_IP6_CONFIG_METHOD_DHCP) ||
-            g_str_equal (method, NM_SETTING_IP6_CONFIG_METHOD_MANUAL))
-                routes_children = gtk_container_get_children (GTK_CONTAINER (self->routes_list));
+        add_routes = g_str_equal (method, NM_SETTING_IP6_CONFIG_METHOD_AUTO) ||
+                     g_str_equal (method, NM_SETTING_IP6_CONFIG_METHOD_DHCP) ||
+                     g_str_equal (method, NM_SETTING_IP6_CONFIG_METHOD_MANUAL);
 
-        for (GList *l = routes_children; l; l = l->next) {
-                GtkWidget *row = l->data;
+        for (child = gtk_widget_get_first_child (self->routes_list);
+             add_routes && child;
+             child = gtk_widget_get_next_sibling (child)) {
+                GtkWidget *row = child;
                 CEIPAddressEntry *address_entry;
                 CEIPAddressEntry *gateway_entry;
                 const gchar *text_prefix;
@@ -645,9 +658,9 @@ ui_to_setting (CEPageIP6 *self)
                 if (!address_entry)
                         continue;
 
-                text_prefix = gtk_entry_get_text (GTK_ENTRY (g_object_get_data (G_OBJECT (row), "prefix")));
+                text_prefix = gtk_editable_get_text (GTK_EDITABLE (g_object_get_data (G_OBJECT (row), "prefix")));
                 gateway_entry = CE_IP_ADDRESS_ENTRY (g_object_get_data (G_OBJECT (row), "gateway"));
-                text_metric = gtk_entry_get_text (GTK_ENTRY (g_object_get_data (G_OBJECT (row), "metric")));
+                text_metric = gtk_editable_get_text (GTK_EDITABLE (g_object_get_data (G_OBJECT (row), "metric")));
 
                 if (ce_ip_address_entry_is_empty (address_entry) && !*text_prefix && ce_ip_address_entry_is_empty (gateway_entry) && !*text_metric) {
                         /* ignore empty rows */
@@ -687,11 +700,16 @@ ui_to_setting (CEPageIP6 *self)
                 if (!ret)
                         continue;
 
-                route = nm_ip_route_new (AF_INET6, gtk_entry_get_text (GTK_ENTRY (address_entry)), prefix, gtk_entry_get_text (GTK_ENTRY (gateway_entry)), metric, NULL);
+                route = nm_ip_route_new (AF_INET6,
+                                         gtk_editable_get_text (GTK_EDITABLE (address_entry)),
+                                         prefix,
+                                         gtk_editable_get_text (GTK_EDITABLE (gateway_entry)),
+                                         metric,
+                                         NULL);
                 nm_setting_ip_config_add_route (self->setting, route);
                 nm_ip_route_unref (route);
 
-                if (!l || !l->next)
+                if (!gtk_widget_get_next_sibling (row))
                         ensure_empty_routes_row (self);
         }
 
@@ -700,7 +718,7 @@ ui_to_setting (CEPageIP6 *self)
 
         ignore_auto_dns = !gtk_switch_get_active (self->auto_dns_switch);
         ignore_auto_routes = !gtk_switch_get_active (self->auto_routes_switch);
-        never_default = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->never_default_check));
+        never_default = gtk_check_button_get_active (self->never_default_check);
 
         g_object_set (self->setting,
                       NM_SETTING_IP_CONFIG_METHOD, method,
@@ -747,6 +765,9 @@ ce_page_ip6_class_init (CEPageIP6Class *klass)
         gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/control-center/network/ip6-page.ui");
 
         gtk_widget_class_bind_template_child (widget_class, CEPageIP6, address_box);
+        gtk_widget_class_bind_template_child (widget_class, CEPageIP6, address_address_label);
+        gtk_widget_class_bind_template_child (widget_class, CEPageIP6, address_prefix_label);
+        gtk_widget_class_bind_template_child (widget_class, CEPageIP6, address_gateway_label);
         gtk_widget_class_bind_template_child (widget_class, CEPageIP6, address_sizegroup);
         gtk_widget_class_bind_template_child (widget_class, CEPageIP6, auto_dns_switch);
         gtk_widget_class_bind_template_child (widget_class, CEPageIP6, auto_routes_switch);
@@ -756,9 +777,14 @@ ce_page_ip6_class_init (CEPageIP6Class *klass)
         gtk_widget_class_bind_template_child (widget_class, CEPageIP6, disabled_radio);
         gtk_widget_class_bind_template_child (widget_class, CEPageIP6, dns_entry);
         gtk_widget_class_bind_template_child (widget_class, CEPageIP6, local_radio);
+        gtk_widget_class_bind_template_child (widget_class, CEPageIP6, main_box);
         gtk_widget_class_bind_template_child (widget_class, CEPageIP6, manual_radio);
         gtk_widget_class_bind_template_child (widget_class, CEPageIP6, never_default_check);
         gtk_widget_class_bind_template_child (widget_class, CEPageIP6, routes_box);
+        gtk_widget_class_bind_template_child (widget_class, CEPageIP6, routes_address_label);
+        gtk_widget_class_bind_template_child (widget_class, CEPageIP6, routes_prefix_label);
+        gtk_widget_class_bind_template_child (widget_class, CEPageIP6, routes_gateway_label);
+        gtk_widget_class_bind_template_child (widget_class, CEPageIP6, routes_metric_label);
         gtk_widget_class_bind_template_child (widget_class, CEPageIP6, routes_metric_sizegroup);
         gtk_widget_class_bind_template_child (widget_class, CEPageIP6, routes_sizegroup);
         gtk_widget_class_bind_template_child (widget_class, CEPageIP6, shared_radio);
