@@ -47,7 +47,7 @@ static void
 show_toggled_cb (EAPMethodLEAP *self)
 {
 	gboolean visible;
-	visible = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->show_password_check));
+	visible = gtk_check_button_get_active (GTK_CHECK_BUTTON (self->show_password_check));
 	gtk_entry_set_visibility (self->password_entry, visible);
 }
 
@@ -55,10 +55,11 @@ static gboolean
 validate (EAPMethod *parent, GError **error)
 {
 	EAPMethodLEAP *self = (EAPMethodLEAP *)parent;
+	NMSettingSecretFlags secret_flags;
 	const char *text;
 	gboolean ret = TRUE;
 
-	text = gtk_entry_get_text (self->username_entry);
+	text = gtk_editable_get_text (GTK_EDITABLE (self->username_entry));
 	if (!text || !strlen (text)) {
 		widget_set_error (GTK_WIDGET (self->username_entry));
 		g_set_error_literal (error, NMA_ERROR, NMA_ERROR_GENERIC, _("missing EAP-LEAP username"));
@@ -66,7 +67,13 @@ validate (EAPMethod *parent, GError **error)
 	} else
 		widget_unset_error (GTK_WIDGET (self->username_entry));
 
-	text = gtk_entry_get_text (self->password_entry);
+	secret_flags = nma_utils_menu_to_secret_flags (GTK_WIDGET (self->password_entry));
+	if (secret_flags & NM_SETTING_SECRET_FLAG_NOT_SAVED) {
+		widget_unset_error (GTK_WIDGET (self->password_entry));
+		return TRUE;
+	}
+
+	text = gtk_editable_get_text (GTK_EDITABLE (self->password_entry));
 	if (!text || !strlen (text)) {
 		widget_set_error (GTK_WIDGET (self->password_entry));
 		if (ret) {
@@ -99,8 +106,8 @@ fill_connection (EAPMethod *parent, NMConnection *connection, NMSettingSecretFla
 
 	nm_setting_802_1x_add_eap_method (s_8021x, "leap");
 
-	g_object_set (s_8021x, NM_SETTING_802_1X_IDENTITY, gtk_entry_get_text (self->username_entry), NULL);
-	g_object_set (s_8021x, NM_SETTING_802_1X_PASSWORD, gtk_entry_get_text (self->password_entry), NULL);
+	g_object_set (s_8021x, NM_SETTING_802_1X_IDENTITY, gtk_editable_get_text (GTK_EDITABLE (self->username_entry)), NULL);
+	g_object_set (s_8021x, NM_SETTING_802_1X_PASSWORD, gtk_editable_get_text (GTK_EDITABLE (self->password_entry)), NULL);
 
 	/* Save 802.1X password flags to the connection */
 	secret_flags = nma_utils_menu_to_secret_flags (GTK_WIDGET (self->password_entry));
@@ -139,42 +146,42 @@ static const gchar *
 get_username (EAPMethod *method)
 {
 	EAPMethodLEAP *self = EAP_METHOD_LEAP (method);
-	return gtk_entry_get_text (self->username_entry);
+	return gtk_editable_get_text (GTK_EDITABLE (self->username_entry));
 }
 
 static void
 set_username (EAPMethod *method, const gchar *username)
 {
 	EAPMethodLEAP *self = EAP_METHOD_LEAP (method);
-	gtk_entry_set_text (self->username_entry, username);
+	gtk_editable_set_text (GTK_EDITABLE (self->username_entry), username);
 }
 
 static const gchar *
 get_password (EAPMethod *method)
 {
 	EAPMethodLEAP *self = EAP_METHOD_LEAP (method);
-	return gtk_entry_get_text (self->password_entry);
+	return gtk_editable_get_text (GTK_EDITABLE (self->password_entry));
 }
 
 static void
 set_password (EAPMethod *method, const gchar *password)
 {
 	EAPMethodLEAP *self = EAP_METHOD_LEAP (method);
-	gtk_entry_set_text (self->password_entry, password);
+	gtk_editable_set_text (GTK_EDITABLE (self->password_entry), password);
 }
 
 static gboolean
 get_show_password (EAPMethod *method)
 {
 	EAPMethodLEAP *self = EAP_METHOD_LEAP (method);
-	return gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->show_password_check));
+	return gtk_check_button_get_active (GTK_CHECK_BUTTON (self->show_password_check));
 }
 
 static void
 set_show_password (EAPMethod *method, gboolean show_password)
 {
 	EAPMethodLEAP *self = EAP_METHOD_LEAP (method);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->show_password_check), show_password);
+	gtk_check_button_set_active (self->show_password_check, show_password);
 }
 
 static void
@@ -208,7 +215,7 @@ eap_method_leap_class_init (EAPMethodLEAPClass *klass)
 
 	object_class->dispose = eap_method_leap_dispose;
 
-	gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/ControlCenter/network/eap-method-leap.ui");
+	gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/Settings/network/eap-method-leap.ui");
 
 	gtk_widget_class_bind_template_child (widget_class, EAPMethodLEAP, password_entry);
 	gtk_widget_class_bind_template_child (widget_class, EAPMethodLEAP, password_label);
