@@ -21,38 +21,63 @@
  * Copyright 2007 - 2014 Red Hat, Inc.
  */
 
-#include "nm-default.h"
-
-#include <string.h>
-#include <sys/types.h>
-#include <sys/stat.h>
 #include <fcntl.h>
-#include <unistd.h>
+#include <glib/gi18n.h>
 
 #include "eap-method.h"
-#include "utils.h"
 #include "helpers.h"
+#include "ui-helpers.h"
 
-GType
-eap_method_get_type (void)
+G_DEFINE_INTERFACE (EAPMethod, eap_method, G_TYPE_OBJECT)
+
+enum {
+        CHANGED,
+        LAST_SIGNAL
+};
+
+static guint signals[LAST_SIGNAL] = { 0 };
+
+static void
+eap_method_default_init (EAPMethodInterface *iface)
 {
-       static GType type_id = 0;
-
-       if (!type_id) {
-               type_id = g_boxed_type_register_static ("CcEAPMethod",
-                                                       (GBoxedCopyFunc) eap_method_ref,
-                                                       (GBoxedFreeFunc) eap_method_unref);
-       }
-
-       return type_id;
+        signals[CHANGED] =
+                g_signal_new ("changed",
+                              G_TYPE_FROM_INTERFACE (iface),
+                              G_SIGNAL_RUN_FIRST,
+                              0,
+                              NULL, NULL,
+                              g_cclosure_marshal_VOID__VOID,
+                              G_TYPE_NONE, 0);
 }
 
 GtkWidget *
-eap_method_get_widget (EAPMethod *self)
+eap_method_get_default_field (EAPMethod *self)
 {
 	g_return_val_if_fail (self != NULL, NULL);
 
-	return self->ui_widget;
+	return EAP_METHOD_GET_IFACE (self)->get_default_field (self);
+}
+
+const gchar *
+eap_method_get_password_flags_name (EAPMethod *self)
+{
+	g_return_val_if_fail (self != NULL, NULL);
+
+	if (EAP_METHOD_GET_IFACE (self)->get_password_flags_name)
+		return EAP_METHOD_GET_IFACE (self)->get_password_flags_name (self);
+	else
+		return NULL;
+}
+
+gboolean
+eap_method_get_phase2 (EAPMethod *self)
+{
+	g_return_val_if_fail (self != NULL, FALSE);
+
+	if (EAP_METHOD_GET_IFACE (self)->get_phase2)
+		return EAP_METHOD_GET_IFACE (self)->get_phase2 (self);
+	else
+		return FALSE;
 }
 
 gboolean
@@ -62,21 +87,28 @@ eap_method_validate (EAPMethod *self, GError **error)
 
 	g_return_val_if_fail (self != NULL, FALSE);
 
-	g_assert (self->validate);
-	result = (*(self->validate)) (self, error);
+	result = (*(EAP_METHOD_GET_IFACE (self)->validate)) (self, error);
 	if (!result && error && !*error)
 		g_set_error_literal (error, NMA_ERROR, NMA_ERROR_GENERIC, _("undefined error in 802.1X security (wpa-eap)"));
 	return result;
 }
 
+void
+eap_method_update_secrets (EAPMethod *self, NMConnection *connection)
+{
+	g_return_if_fail (self != NULL);
+
+	if (EAP_METHOD_GET_IFACE (self)->update_secrets)
+		EAP_METHOD_GET_IFACE (self)->update_secrets (self, connection);
+}
+
 void
 eap_method_add_to_size_group (EAPMethod *self, GtkSizeGroup *group)
 {
 	g_return_if_fail (self != NULL);
 	g_return_if_fail (group != NULL);
 
-	g_assert (self->add_to_size_group);
-	return (*(self->add_to_size_group)) (self, group);
+	return (*(EAP_METHOD_GET_IFACE (self)->add_to_size_group)) (self, group);
 }
 
 void
@@ -87,124 +119,57 @@ eap_method_fill_connection (EAPMethod *self,
 	g_return_if_fail (self != NULL);
 	g_return_if_fail (connection != NULL);
 
-	g_assert (self->fill_connection);
-	return (*(self->fill_connection)) (self, connection, flags);
+	return (*(EAP_METHOD_GET_IFACE (self)->fill_connection)) (self, connection, flags);
 }
 
 void
-eap_method_update_secrets (EAPMethod *self, NMConnection *connection)
+eap_method_emit_changed (EAPMethod *self)
 {
-	g_return_if_fail (self != NULL);
-	g_return_if_fail (connection != NULL);
+        g_return_if_fail (EAP_IS_METHOD (self));
 
-	if (self->update_secrets)
-		self->update_secrets (self, connection);
+        g_signal_emit (self, signals[CHANGED], 0);
+}
+
+const gchar *
+eap_method_get_username (EAPMethod *self)
+{
+	g_return_val_if_fail (EAP_IS_METHOD (self), NULL);
+	return EAP_METHOD_GET_IFACE (self)->get_username (self);
 }
 
 void
-eap_method_phase2_update_secrets_helper (EAPMethod *self,
-                                         NMConnection *connection,
-                                         GtkComboBox *combo,
-                                         guint32 column)
+eap_method_set_username (EAPMethod *self, const gchar *username)
 {
-	GtkTreeIter iter;
-	GtkTreeModel *model;
-
-	g_return_if_fail (self != NULL);
-	g_return_if_fail (connection != NULL);
-	g_return_if_fail (combo != NULL);
-
-	/* Let each EAP phase2 method try to update its secrets */
-	model = gtk_combo_box_get_model (combo);
-	if (gtk_tree_model_get_iter_first (model, &iter)) {
-		do {
-			g_autoptr(EAPMethod) eap = NULL;
-
-			gtk_tree_model_get (model, &iter, column, &eap, -1);
-			if (eap)
-				eap_method_update_secrets (eap, connection);
-		} while (gtk_tree_model_iter_next (model, &iter));
-	}
+	g_return_if_fail (EAP_IS_METHOD (self));
+	EAP_METHOD_GET_IFACE (self)->set_username (self, username);
 }
 
-EAPMethod *
-eap_method_init (gsize obj_size,
-                 EMValidateFunc validate,
-                 EMAddToSizeGroupFunc add_to_size_group,
-                 EMFillConnectionFunc fill_connection,
-                 EMUpdateSecretsFunc update_secrets,
-                 EMDestroyFunc destroy,
-                 const char *ui_resource,
-                 const char *ui_widget_name,
-                 const char *default_field,
-                 gboolean phase2)
+const gchar *
+eap_method_get_password (EAPMethod *self)
 {
-	g_autoptr(EAPMethod) self = NULL;
-	g_autoptr(GError) error = NULL;
-
-	g_return_val_if_fail (obj_size > 0, NULL);
-	g_return_val_if_fail (ui_resource != NULL, NULL);
-	g_return_val_if_fail (ui_widget_name != NULL, NULL);
-
-	self = g_slice_alloc0 (obj_size);
-	g_assert (self);
-
-	self->refcount = 1;
-	self->obj_size = obj_size;
-	self->validate = validate;
-	self->add_to_size_group = add_to_size_group;
-	self->fill_connection = fill_connection;
-	self->update_secrets = update_secrets;
-	self->default_field = default_field;
-	self->phase2 = phase2;
-
-	self->builder = gtk_builder_new ();
-	if (!gtk_builder_add_from_resource (self->builder, ui_resource, &error)) {
-		g_warning ("Couldn't load UI builder resource %s: %s",
-		           ui_resource, error->message);
-		return NULL;
-	}
-
-	self->ui_widget = GTK_WIDGET (gtk_builder_get_object (self->builder, ui_widget_name));
-	if (!self->ui_widget) {
-		g_warning ("Couldn't load UI widget '%s' from UI file %s",
-		           ui_widget_name, ui_resource);
-		return NULL;
-	}
-	g_object_ref_sink (self->ui_widget);
-
-	self->destroy = destroy;
-
-	return g_steal_pointer (&self);
-}
-
-
-EAPMethod *
-eap_method_ref (EAPMethod *self)
-{
-	g_return_val_if_fail (self != NULL, NULL);
-	g_return_val_if_fail (self->refcount > 0, NULL);
-
-	self->refcount++;
-	return self;
+	g_return_val_if_fail (EAP_IS_METHOD (self), NULL);
+	return EAP_METHOD_GET_IFACE (self)->get_password (self);
 }
 
 void
-eap_method_unref (EAPMethod *self)
+eap_method_set_password (EAPMethod *self, const gchar *password)
 {
-	g_return_if_fail (self != NULL);
-	g_return_if_fail (self->refcount > 0);
+	g_return_if_fail (EAP_IS_METHOD (self));
+	EAP_METHOD_GET_IFACE (self)->set_password (self, password);
+}
 
-	self->refcount--;
-	if (self->refcount == 0) {
-		if (self->destroy)
-			self->destroy (self);
+gboolean
+eap_method_get_show_password (EAPMethod *self)
+{
+	g_return_val_if_fail (EAP_IS_METHOD (self), FALSE);
+	return EAP_METHOD_GET_IFACE (self)->get_show_password (self);
+}
 
-		g_clear_object (&self->builder);
-		g_clear_object (&self->ui_widget);
-
-		g_slice_free1 (self->obj_size, self);
-	}
+void
+eap_method_set_show_password (EAPMethod *self, gboolean show_password)
+{
+	g_return_if_fail (EAP_IS_METHOD (self));
+	EAP_METHOD_GET_IFACE (self)->set_show_password (self, show_password);
 }
 
 gboolean
@@ -216,6 +181,7 @@ eap_method_validate_filepicker (GtkFileChooser *chooser,
 {
 	g_autofree gchar *filename = NULL;
 	g_autoptr(NMSetting8021x) setting = NULL;
+	g_autoptr(GFile) file = NULL;
 	gboolean success = TRUE;
 
 	if (item_type == TYPE_PRIVATE_KEY) {
@@ -223,8 +189,8 @@ eap_method_validate_filepicker (GtkFileChooser *chooser,
 			success = FALSE;
 	}
 
-	filename = gtk_file_chooser_get_filename (chooser);
-	if (!filename) {
+	file = gtk_file_chooser_get_file (chooser);
+	if (!file) {
 		if (item_type != TYPE_CA_CERT) {
 			success = FALSE;
 			g_set_error_literal (error, NMA_ERROR, NMA_ERROR_GENERIC, _("no file selected"));
@@ -232,6 +198,7 @@ eap_method_validate_filepicker (GtkFileChooser *chooser,
 		goto out;
 	}
 
+	filename = g_file_get_path (file);
 	if (!g_file_test (filename, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR)) {
 		success = FALSE;
 		goto out;
@@ -264,23 +231,28 @@ out:
 }
 
 static gboolean
-file_has_extension (const char *filename, const char *extensions[])
+file_is_mime_type (const char *path, const char *mime_types[])
 {
-	char *p;
-	g_autofree gchar *ext = NULL;
+	g_autoptr(GFile) file = NULL;
+	g_autoptr(GFileInfo) info = NULL;
+	const char *content_type;
 	int i = 0;
 	gboolean found = FALSE;
 
-	p = strrchr (filename, '.');
-	if (!p)
-		return FALSE;
-
-	ext = g_ascii_strdown (p, -1);
-	if (ext) {
-		while (extensions[i]) {
-			if (!strcmp (ext, extensions[i++])) {
-				found = TRUE;
-				break;
+	file = g_file_new_for_path (path);
+	info = g_file_query_info (file, 
+	                          G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
+	                          G_FILE_QUERY_INFO_NONE,
+	                          NULL,
+	                          NULL);
+	if (info) {
+		content_type = g_file_info_get_content_type (info);
+		if (content_type) {
+			while (mime_types[i]) {
+				if (strcmp (content_type, mime_types[i++]) == 0) {
+					found = TRUE;
+					break;
+				}
 			}
 		}
 	}
@@ -393,33 +365,23 @@ out:
 }
 #endif
 
-static gboolean
-default_filter_privkey (const GtkFileFilterInfo *filter_info, gpointer user_data)
+static const char *privkey_mime_types[] = {
+	"application/x-x509-ca-cert", "application/pkcs12", "application/x-pkcs12", "application/pgp-keys", NULL
+};
+static const char *cert_mime_types[] = {
+	"application/x-x509-ca-cert", NULL
+};
+
+static void
+add_mime_types_to_filter (GtkFileFilter  *filter,
+                          const char    **mime_types)
 {
-	const char *extensions[] = { ".der", ".pem", ".p12", ".key", NULL };
+	int i;
 
-	if (!filter_info->filename)
-		return FALSE;
-
-	if (!file_has_extension (filter_info->filename, extensions))
-		return FALSE;
-
-	return TRUE;
+	for (i = 0; mime_types[i] != NULL; i++)
+		gtk_file_filter_add_mime_type (filter, mime_types[i]);
 }
 
-static gboolean
-default_filter_cert (const GtkFileFilterInfo *filter_info, gpointer user_data)
-{
-	const char *extensions[] = { ".der", ".pem", ".crt", ".cer", NULL };
-
-	if (!filter_info->filename)
-		return FALSE;
-
-	if (!file_has_extension (filter_info->filename, extensions))
-		return FALSE;
-
-	return TRUE;
-}
 
 GtkFileFilter *
 eap_method_default_file_chooser_filter_new (gboolean privkey)
@@ -428,11 +390,11 @@ eap_method_default_file_chooser_filter_new (gboolean privkey)
 
 	filter = gtk_file_filter_new ();
 	if (privkey) {
-		gtk_file_filter_add_custom (filter, GTK_FILE_FILTER_FILENAME, default_filter_privkey, NULL, NULL);
-		gtk_file_filter_set_name (filter, _("DER, PEM, or PKCS#12 private keys (*.der, *.pem, *.p12, *.key)"));
+		add_mime_types_to_filter (filter, privkey_mime_types);
+		gtk_file_filter_set_name (filter, _("DER, PEM, PKCS#12, or PGP private keys"));
 	} else {
-		gtk_file_filter_add_custom (filter, GTK_FILE_FILTER_FILENAME, default_filter_cert, NULL, NULL);
-		gtk_file_filter_set_name (filter, _("DER or PEM certificates (*.der, *.pem, *.crt, *.cer)"));
+		add_mime_types_to_filter (filter, cert_mime_types);
+		gtk_file_filter_set_name (filter, _("DER or PEM certificates"));
 	}
 	return filter;
 }
@@ -440,10 +402,9 @@ eap_method_default_file_chooser_filter_new (gboolean privkey)
 gboolean
 eap_method_is_encrypted_private_key (const char *path)
 {
-	GtkFileFilterInfo info = { .filename = path };
 	gboolean is_encrypted;
 
-	if (!default_filter_privkey (&info, NULL))
+	if (!file_is_mime_type (path, privkey_mime_types))
 		return FALSE;
 
 #if LIBNM_BUILD
@@ -459,53 +420,30 @@ eap_method_is_encrypted_private_key (const char *path)
 	return is_encrypted;
 }
 
-/* Some methods (PEAP, TLS, TTLS) require a CA certificate. The user can choose
- * not to provide such a certificate. This method whether the checkbox
- * id_ca_cert_not_required_checkbutton is checked or id_ca_cert_chooser has a certificate
- * selected.
- */
-gboolean
-eap_method_ca_cert_required (GtkToggleButton *id_ca_cert_not_required_checkbutton, GtkFileChooser *id_ca_cert_chooser)
-{
-	g_assert (id_ca_cert_not_required_checkbutton && id_ca_cert_chooser);
-
-	if (!gtk_toggle_button_get_active (id_ca_cert_not_required_checkbutton)) {
-		g_autofree gchar *filename = NULL;
-
-		filename = gtk_file_chooser_get_filename (id_ca_cert_chooser);
-		if (!filename)
-			return TRUE;
-	}
-	return FALSE;
-}
-
-
 void
-eap_method_ca_cert_not_required_toggled (GtkToggleButton *id_ca_cert_not_required_checkbutton, GtkFileChooser *id_ca_cert_chooser)
+eap_method_ca_cert_not_required_toggled (GtkCheckButton *id_ca_cert_not_required_checkbutton, GtkFileChooser *id_ca_cert_chooser)
 {
-	g_autofree gchar *filename = NULL;
-	g_autofree gchar *filename_old = NULL;
+	g_autoptr(GFile) file = NULL;
+	g_autoptr(GFile) file_old = NULL;
 	gboolean is_not_required;
 
 	g_assert (id_ca_cert_not_required_checkbutton && id_ca_cert_chooser);
 
-	is_not_required = gtk_toggle_button_get_active (id_ca_cert_not_required_checkbutton);
+	is_not_required = gtk_check_button_get_active (id_ca_cert_not_required_checkbutton);
 
-	filename = gtk_file_chooser_get_filename (id_ca_cert_chooser);
-	filename_old = g_object_steal_data (G_OBJECT (id_ca_cert_chooser), "filename-old");
+	file = gtk_file_chooser_get_file (id_ca_cert_chooser);
+	file_old = g_object_steal_data (G_OBJECT (id_ca_cert_chooser), "filename-old");
 	if (is_not_required) {
-		g_free (filename_old);
-		filename_old = g_steal_pointer (&filename);
+		g_clear_object (&file_old);
+		file_old = g_steal_pointer (&file);
 	} else {
-		g_free (filename);
-		filename = g_steal_pointer (&filename_old);
+		g_clear_object (&file);
+		file = g_steal_pointer (&file_old);
 	}
 	gtk_widget_set_sensitive (GTK_WIDGET (id_ca_cert_chooser), !is_not_required);
-	if (filename)
-		gtk_file_chooser_set_filename (id_ca_cert_chooser, filename);
-	else
-		gtk_file_chooser_unselect_all (id_ca_cert_chooser);
-	g_object_set_data_full (G_OBJECT (id_ca_cert_chooser), "filename-old", g_steal_pointer (&filename_old), g_free);
+	if (file)
+		gtk_file_chooser_set_file (id_ca_cert_chooser, file, NULL);
+	g_object_set_data_full (G_OBJECT (id_ca_cert_chooser), "filename-old", g_steal_pointer (&file_old), g_free);
 }
 
 /* Used as both GSettings keys and GObject data tags */
@@ -539,7 +477,7 @@ eap_method_ca_cert_ignore_set (EAPMethod *self,
 	if (s_8021x) {
 		ignore = !ca_cert_error && filename == NULL;
 		g_object_set_data (G_OBJECT (s_8021x),
-		                   self->phase2 ? IGNORE_PHASE2_CA_CERT_TAG : IGNORE_CA_CERT_TAG,
+		                   eap_method_get_phase2 (self) ? IGNORE_PHASE2_CA_CERT_TAG : IGNORE_CA_CERT_TAG,
 		                   GUINT_TO_POINTER (ignore));
 	}
 }
@@ -560,7 +498,7 @@ eap_method_ca_cert_ignore_get (EAPMethod *self, NMConnection *connection)
 	s_8021x = nm_connection_get_setting_802_1x (connection);
 	if (s_8021x) {
 		return !!g_object_get_data (G_OBJECT (s_8021x),
-		                            self->phase2 ? IGNORE_PHASE2_CA_CERT_TAG : IGNORE_CA_CERT_TAG);
+		                            eap_method_get_phase2 (self) ? IGNORE_PHASE2_CA_CERT_TAG : IGNORE_CA_CERT_TAG);
 	}
 	return FALSE;
 }
