@@ -31,10 +31,10 @@
 
 #include <cups/cups.h>
 
-#include "list-box-helper.h"
 #include "pp-jobs-dialog.h"
 #include "pp-utils.h"
 #include "pp-job.h"
+#include "pp-job-row.h"
 #include "pp-cups.h"
 #include "pp-printer.h"
 
@@ -69,6 +69,7 @@ struct _PpJobsDialog {
   gchar    **actual_auth_info_required;
   gboolean   jobs_filled;
   gboolean   pop_up_authentication_popup;
+  gint       max_priority;
 
   GCancellable *get_jobs_cancellable;
 };
@@ -142,7 +143,7 @@ static void
 auth_entries_activated (PpJobsDialog *self)
 {
   if (auth_popup_filled (self))
-    gtk_button_clicked (self->authenticate_button);
+    g_signal_emit_by_name (self->authenticate_button, "activate");
 }
 
 static void
@@ -159,133 +160,69 @@ authenticate_popover_update (PpJobsDialog *self)
   gtk_widget_set_visible (GTK_WIDGET (self->domain_label), domain_required);
   gtk_widget_set_visible (GTK_WIDGET (self->domain_entry), domain_required);
   if (domain_required)
-    gtk_entry_set_text (self->domain_entry, "");
+    gtk_editable_set_text (GTK_EDITABLE (self->domain_entry), "");
 
   gtk_widget_set_visible (GTK_WIDGET (self->username_label), username_required);
   gtk_widget_set_visible (GTK_WIDGET (self->username_entry), username_required);
   if (username_required)
-    gtk_entry_set_text (self->username_entry, cupsUser ());
+    gtk_editable_set_text (GTK_EDITABLE (self->username_entry), cupsUser ());
 
   gtk_widget_set_visible (GTK_WIDGET (self->password_label), password_required);
   gtk_widget_set_visible (GTK_WIDGET (self->password_entry), password_required);
   if (password_required)
-    gtk_entry_set_text (self->password_entry, "");
+    gtk_editable_set_text (GTK_EDITABLE (self->password_entry), "");
 
   gtk_widget_set_sensitive (GTK_WIDGET (self->authenticate_button), FALSE);
 }
 
 static void
-job_stop_cb (GtkButton *button,
-             PpJob     *job)
+pp_job_update_cb (GObject      *source_object,
+                  GAsyncResult *res,
+                  gpointer      user_data)
 {
-  pp_job_cancel_purge_async (job, FALSE);
+  PpJobsDialog     *self = user_data;
+  gboolean          result;
+  g_autoptr(GError) error = NULL;
+  PpJob            *job = PP_JOB (source_object);
+
+  result = pp_job_set_priority_finish (job, res, &error);
+  if (result)
+    {
+      pp_jobs_dialog_update (self);
+    }
+  else if (error != NULL)
+    {
+      if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+        {
+          g_warning ("Could not set job priority: %s", error->message);
+        }
+    }
 }
 
 static void
-job_pause_cb (GtkButton *button,
-              PpJob     *job)
+on_priority_changed (PpJobRow     *job_row,
+                     PpJobsDialog *self)
 {
-  gint job_state;
+  PpJob *job;
 
-  g_object_get (job, "state", &job_state, NULL);
-
-  pp_job_set_hold_until_async (job, job_state == IPP_JOB_HELD ? "no-hold" : "indefinite");
-
-  gtk_button_set_image (button,
-                        gtk_image_new_from_icon_name (job_state == IPP_JOB_HELD ?
-                                                      "media-playback-pause-symbolic" : "media-playback-start-symbolic",
-                                                      GTK_ICON_SIZE_SMALL_TOOLBAR));
+  job = pp_job_row_get_job (job_row);
+  pp_job_set_priority_async (job, ++self->max_priority, NULL, pp_job_update_cb, self);
 }
 
 static GtkWidget *
 create_listbox_row (gpointer item,
                     gpointer user_data)
 {
-  GtkWidget  *widget;
-  GtkWidget  *box;
-  PpJob      *job = (PpJob *)item;
-  gchar     **auth_info_required;
-  gchar      *title;
-  gchar      *state_string = NULL;
-  gint        job_state;
+  PpJobRow *job_row;
 
-  g_object_get (job,
-                "title", &title,
-                "state", &job_state,
-                "auth-info-required", &auth_info_required,
-                NULL);
+  job_row = pp_job_row_new (PP_JOB (item));
 
-  switch (job_state)
-    {
-      case IPP_JOB_PENDING:
-        /* Translators: Job's state (job is waiting to be printed) */
-        state_string = g_strdup (C_("print job", "Pending"));
-        break;
-      case IPP_JOB_HELD:
-        if (auth_info_required == NULL)
-          {
-            /* Translators: Job's state (job is held for printing) */
-            state_string = g_strdup (C_("print job", "Paused"));
-          }
-        else
-          {
-            /* Translators: Job's state (job needs authentication to proceed further) */
-            state_string = g_strdup_printf ("<span foreground=\"#ff0000\">%s</span>", C_("print job", "Authentication required"));
-          }
-        break;
-      case IPP_JOB_PROCESSING:
-        /* Translators: Job's state (job is currently printing) */
-        state_string = g_strdup (C_("print job", "Processing"));
-        break;
-      case IPP_JOB_STOPPED:
-        /* Translators: Job's state (job has been stopped) */
-        state_string = g_strdup (C_("print job", "Stopped"));
-        break;
-      case IPP_JOB_CANCELED:
-        /* Translators: Job's state (job has been canceled) */
-        state_string = g_strdup (C_("print job", "Canceled"));
-        break;
-      case IPP_JOB_ABORTED:
-        /* Translators: Job's state (job has aborted due to error) */
-        state_string = g_strdup (C_("print job", "Aborted"));
-        break;
-      case IPP_JOB_COMPLETED:
-        /* Translators: Job's state (job has completed successfully) */
-        state_string = g_strdup (C_("print job", "Completed"));
-        break;
-    }
+  g_signal_connect (job_row,
+                    "priority-changed",
+                    G_CALLBACK (on_priority_changed),
+                    user_data);
 
-  box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
-  g_object_set (box, "margin", 6, NULL);
-  gtk_container_set_border_width (GTK_CONTAINER (box), 2);
-
-  widget = gtk_label_new (title);
-  gtk_label_set_max_width_chars (GTK_LABEL (widget), 40);
-  gtk_label_set_ellipsize (GTK_LABEL (widget), PANGO_ELLIPSIZE_END);
-  gtk_widget_set_halign (widget, GTK_ALIGN_START);
-  gtk_box_pack_start (GTK_BOX (box), widget, TRUE, TRUE, 10);
-
-  widget = gtk_label_new (NULL);
-  gtk_label_set_markup (GTK_LABEL (widget), state_string);
-  gtk_widget_set_halign (widget, GTK_ALIGN_END);
-  gtk_widget_set_margin_end (widget, 64);
-  gtk_widget_set_margin_start (widget, 64);
-  gtk_box_pack_start (GTK_BOX (box), widget, FALSE, FALSE, 10);
-
-  widget = gtk_button_new_from_icon_name (job_state == IPP_JOB_HELD ? "media-playback-start-symbolic" : "media-playback-pause-symbolic",
-                                          GTK_ICON_SIZE_SMALL_TOOLBAR);
-  g_signal_connect (widget, "clicked", G_CALLBACK (job_pause_cb), item);
-  gtk_widget_set_sensitive (widget, auth_info_required == NULL);
-  gtk_box_pack_start (GTK_BOX (box), widget, FALSE, FALSE, 4);
-
-  widget = gtk_button_new_from_icon_name ("edit-delete-symbolic",
-                                          GTK_ICON_SIZE_SMALL_TOOLBAR);
-  g_signal_connect (widget, "clicked", G_CALLBACK (job_stop_cb), item);
-  gtk_box_pack_start (GTK_BOX (box), widget, FALSE, FALSE, 4);
-
-  gtk_widget_show_all (box);
-
-  return box;
+  return GTK_WIDGET (job_row);
 }
 
 static void
@@ -305,9 +242,12 @@ update_jobs_list_cb (GObject      *source_object,
   g_autoptr(GError)    error = NULL;
   g_autoptr(GPtrArray) jobs;
   PpJob               *job;
-  gchar              **auth_info_required = NULL;
   gint                 num_of_auth_jobs = 0;
+  gint                 job_priority;
+  guint                state;
   guint                i;
+  gint                 current_max_value = 1;
+  gint                 first_unprocessed_job = -1;
 
   g_list_store_remove_all (self->store);
 
@@ -333,28 +273,43 @@ update_jobs_list_cb (GObject      *source_object,
       gtk_stack_set_visible_child (self->stack, GTK_WIDGET (self->no_jobs_page));
     }
 
+  for (i = 0; i < jobs->len; i++)
+  {
+    job = PP_JOB (g_ptr_array_index (jobs, i));
+    state = pp_job_get_state (job);
+
+    if (state == IPP_JOB_PENDING || state == IPP_JOB_HELD)
+      {
+        if (first_unprocessed_job == -1)
+          {
+            first_unprocessed_job = i;
+            break;
+          }
+      }
+  }
+
   for (i = 0; i < jobs->len; i++)
     {
       job = PP_JOB (g_ptr_array_index (jobs, i));
+      job_priority = pp_job_get_priority (job);
+      pp_job_priority_set_sensitive (job, (pp_job_get_state (job) == IPP_JOB_PENDING ||
+                                     pp_job_get_state (job) == IPP_JOB_HELD) &&
+                                     i > first_unprocessed_job);
+
+      if (job_priority >= current_max_value && job_priority != 100)
+        current_max_value = job_priority;
 
       g_list_store_append (self->store, g_object_ref (job));
 
-      g_object_get (G_OBJECT (job),
-                    "auth-info-required", &auth_info_required,
-                    NULL);
-      if (auth_info_required != NULL)
+      if (pp_job_get_auth_info_required (job) != NULL)
         {
           num_of_auth_jobs++;
 
           if (self->actual_auth_info_required == NULL)
-            self->actual_auth_info_required = auth_info_required;
-          else
-            g_strfreev (auth_info_required);
-
-          auth_info_required = NULL;
+            self->actual_auth_info_required = g_strdupv (pp_job_get_auth_info_required (job));
         }
     }
-
+  self->max_priority = current_max_value;
   if (num_of_auth_jobs > 0)
     {
       g_autofree gchar *text = NULL;
@@ -389,7 +344,7 @@ update_jobs_list_cb (GObject      *source_object,
 static void
 update_jobs_list (PpJobsDialog *self)
 {
-  PpPrinter *printer;
+  g_autoptr(PpPrinter) printer = NULL;
 
   if (self->printer_name != NULL)
     {
@@ -452,7 +407,6 @@ static void
 authenticate_button_clicked (PpJobsDialog *self)
 {
   PpJob        *job;
-  gchar       **auth_info_required = NULL;
   gchar       **auth_info;
   guint         num_items;
   gint          i;
@@ -461,11 +415,11 @@ authenticate_button_clicked (PpJobsDialog *self)
   for (i = 0; self->actual_auth_info_required[i] != NULL; i++)
     {
       if (g_strcmp0 (self->actual_auth_info_required[i], "domain") == 0)
-        auth_info[i] = g_strdup (gtk_entry_get_text (GTK_ENTRY (self->domain_entry)));
+        auth_info[i] = g_strdup (gtk_editable_get_text (GTK_EDITABLE (self->domain_entry)));
       else if (g_strcmp0 (self->actual_auth_info_required[i], "username") == 0)
-        auth_info[i] = g_strdup (gtk_entry_get_text (GTK_ENTRY (self->username_entry)));
+        auth_info[i] = g_strdup (gtk_editable_get_text (GTK_EDITABLE (self->username_entry)));
       else if (g_strcmp0 (self->actual_auth_info_required[i], "password") == 0)
-        auth_info[i] = g_strdup (gtk_entry_get_text (GTK_ENTRY (self->password_entry)));
+        auth_info[i] = g_strdup (gtk_editable_get_text (GTK_EDITABLE (self->password_entry)));
     }
 
   num_items = g_list_model_get_n_items (G_LIST_MODEL (self->store));
@@ -473,30 +427,15 @@ authenticate_button_clicked (PpJobsDialog *self)
     {
       job = PP_JOB (g_list_model_get_item (G_LIST_MODEL (self->store), i));
 
-      g_object_get (job, "auth-info-required", &auth_info_required, NULL);
-      if (auth_info_required != NULL)
+      if (pp_job_get_auth_info_required (job) != NULL)
         {
           pp_job_authenticate_async (job, auth_info, NULL, pp_job_authenticate_cb, self);
-
-          g_strfreev (auth_info_required);
-          auth_info_required = NULL;
         }
     }
 
   g_strfreev (auth_info);
 }
 
-static gboolean
-key_press_event_cb (GtkWidget   *widget,
-                    GdkEventKey *event,
-                    gpointer     user_data)
-{
-  if (event->keyval == GDK_KEY_Escape)
-    gtk_dialog_response (GTK_DIALOG (widget), GTK_RESPONSE_CLOSE);
-
-  return FALSE;
-}
-
 PpJobsDialog *
 pp_jobs_dialog_new (const gchar *printer_name)
 {
@@ -513,9 +452,6 @@ pp_jobs_dialog_new (const gchar *printer_name)
   self->jobs_filled = FALSE;
   self->pop_up_authentication_popup = FALSE;
 
-  /* connect signals */
-  g_signal_connect (self, "key-press-event", G_CALLBACK (key_press_event_cb), NULL);
-
   /* Translators: This is the printer name for which we are showing the active jobs */
   title = g_strdup_printf (C_("Printer jobs dialog title", "%s — Active Jobs"), printer_name);
   gtk_window_set_title (GTK_WINDOW (self), title);
@@ -524,11 +460,9 @@ pp_jobs_dialog_new (const gchar *printer_name)
   text = g_strdup_printf (_("Enter credentials to print from %s."), printer_name);
   gtk_label_set_text (self->authentication_label, text);
 
-  gtk_list_box_set_header_func (self->jobs_listbox,
-                                cc_list_box_update_header_func, NULL, NULL);
   self->store = g_list_store_new (pp_job_get_type ());
   gtk_list_box_bind_model (self->jobs_listbox, G_LIST_MODEL (self->store),
-                           create_listbox_row, NULL, NULL);
+                           create_listbox_row, self, NULL);
 
   update_jobs_list (self);
 
@@ -600,4 +534,6 @@ pp_jobs_dialog_class_init (PpJobsDialogClass *klass)
   gtk_widget_class_bind_template_callback (widget_class, auth_entries_changed);
 
   object_class->dispose = pp_jobs_dialog_dispose;
+
+  gtk_widget_class_add_binding_action (widget_class, GDK_KEY_Escape, 0, "window.close", NULL);
 }
