@@ -295,44 +295,31 @@ ppd_option_name_translate (ppd_option_t *option)
 static gint
 grid_get_height (GtkWidget *grid)
 {
-  GList *children;
-  GList *child;
+  GtkWidget *child;
   gint   height = 0;
-  gint   top_attach = 0;
+  gint   row = 0;
   gint   max = 0;
 
-  children = gtk_container_get_children (GTK_CONTAINER (grid));
-  for (child = children; child; child = g_list_next (child))
+  for (child = gtk_widget_get_first_child (grid);
+       child;
+       child = gtk_widget_get_next_sibling (child))
     {
-      gtk_container_child_get (GTK_CONTAINER (grid), child->data,
-                               "top-attach", &top_attach,
-                               "height", &height,
-                               NULL);
+      gtk_grid_query_child (GTK_GRID (grid),
+                            child,
+                            NULL, &row,
+                            NULL, &height);
 
-      if (height + top_attach > max)
-        max = height + top_attach;
+      if (height + row > max)
+        max = height + row;
     }
 
-  g_list_free (children);
-
   return max;
 }
 
 static gboolean
 grid_is_empty (GtkWidget *grid)
 {
-  GList *children;
-
-  children = gtk_container_get_children (GTK_CONTAINER (grid));
-  if (children)
-    {
-      g_list_free (children);
-      return FALSE;
-    }
-  else
-    {
-      return TRUE;
-    }
+  return gtk_widget_get_first_child (grid) == NULL;
 }
 
 static GtkWidget *
@@ -355,7 +342,7 @@ ipp_option_add (IPPAttribute *attr_supported,
                                                    printer_name);
   if (widget)
     {
-      gtk_widget_show_all (widget);
+      gtk_widget_show (widget);
       gtk_widget_set_sensitive (widget, sensitive);
       position = grid_get_height (grid);
 
@@ -390,12 +377,11 @@ ppd_option_add (ppd_option_t  option,
   widget = (GtkWidget *) pp_ppd_option_widget_new (&option, printer_name);
   if (widget)
     {
-      gtk_widget_show_all (widget);
+      gtk_widget_show (widget);
       gtk_widget_set_sensitive (widget, sensitive);
       position = grid_get_height (grid);
 
       label = gtk_label_new (ppd_option_name_translate (&option));
-      gtk_widget_show (GTK_WIDGET (label));
       gtk_label_set_mnemonic_widget (GTK_LABEL (label), widget);
       context = gtk_widget_get_style_context (label);
       gtk_style_context_add_class (context, "dim-label");
@@ -417,8 +403,10 @@ tab_grid_new ()
   GtkWidget *grid;
 
   grid = gtk_grid_new ();
-  gtk_widget_show (GTK_WIDGET (grid));
-  gtk_container_set_border_width (GTK_CONTAINER (grid), 20);
+  gtk_widget_set_margin_start (grid, 20);
+  gtk_widget_set_margin_end (grid, 20);
+  gtk_widget_set_margin_top (grid, 20);
+  gtk_widget_set_margin_bottom (grid, 20);
   gtk_grid_set_row_spacing (GTK_GRID (grid), 15);
 
   return grid;
@@ -437,12 +425,11 @@ tab_add (PpOptionsDialog *self,
 
   if (!grid_is_empty (grid))
     {
-      scrolled_window = gtk_scrolled_window_new (NULL, NULL);
-      gtk_widget_show (GTK_WIDGET (scrolled_window));
+      scrolled_window = gtk_scrolled_window_new ();
       gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
                                       GTK_POLICY_NEVER,
                                       GTK_POLICY_AUTOMATIC);
-      gtk_container_add (GTK_CONTAINER (scrolled_window), grid);
+      gtk_scrolled_window_set_child (GTK_SCROLLED_WINDOW (scrolled_window), grid);
 
       id = gtk_notebook_append_page (self->notebook,
                                      scrolled_window,
@@ -727,7 +714,7 @@ get_ipp_attributes_cb (GHashTable *table,
   if (self->ipp_attributes)
     g_hash_table_unref (self->ipp_attributes);
 
-  self->ipp_attributes = table;
+  self->ipp_attributes = g_hash_table_ref (table);
   self->ipp_attributes_set = TRUE;
 
   if (self->ppd_filename_set &&
@@ -789,11 +776,7 @@ pp_maintenance_command_execute_cb (GObject      *source_object,
                                    GAsyncResult *res,
                                    gpointer      user_data)
 {
-  PpMaintenanceCommand *command = (PpMaintenanceCommand *) source_object;
-
-  pp_maintenance_command_execute_finish (command, res, NULL);
-
-  g_object_unref (command);
+  pp_maintenance_command_execute_finish (PP_MAINTENANCE_COMMAND(source_object), res, NULL);
 }
 
 static gchar *
@@ -824,8 +807,6 @@ print_test_page_cb (GObject      *source_object,
 {
   pp_printer_print_file_finish (PP_PRINTER (source_object),
                                 result, NULL);
-
-  g_object_unref (source_object);
 }
 
 static void
@@ -854,7 +835,7 @@ test_page_cb (PpOptionsDialog *self)
 
       if (filename != NULL)
         {
-          PpPrinter *printer;
+          g_autoptr(PpPrinter) printer = NULL;
 
           printer = pp_printer_new (self->printer_name);
           pp_printer_print_file_async (printer,
@@ -867,7 +848,7 @@ test_page_cb (PpOptionsDialog *self)
         }
       else
         {
-          PpMaintenanceCommand *command;
+          g_autoptr(PpMaintenanceCommand) command = NULL;
 
           command = pp_maintenance_command_new (self->printer_name,
                                                 "PrintSelfTestPage",
