@@ -218,6 +218,7 @@ void        printer_get_ppd_async (const gchar *printer_name,
                                    PGPCallback  callback,
                                    gpointer     user_data);
 
+/* NOTE: 'destination' is passed with ownership as cupsCopyDest doesn't seem to work as expected */
 typedef void (*GNDCallback) (cups_dest_t *destination,
                              gpointer     user_data);
 
@@ -257,4 +258,6 @@ gchar      *canonicalize_device_name (GList         *device_names,
 
 void        shift_string_left (gchar *str);
 
+gboolean    printer_name_is_valid (const gchar *str);
+
 G_END_DECLS
