@@ -21,19 +21,19 @@
 
 struct _CcSearchPanelRow
 {
-  GtkListBoxRow  parent_instance;
+  AdwActionRow   parent_instance;
 
   GAppInfo      *app_info;
 
-  GtkEventBox   *drag_handle;
   GtkImage      *icon;
-  GtkLabel      *app_name;
   GtkSwitch     *switcher;
 
   GtkListBox    *drag_widget;
+  gdouble        drag_x;
+  gdouble        drag_y;
 };
 
-G_DEFINE_TYPE (CcSearchPanelRow, cc_search_panel_row, GTK_TYPE_LIST_BOX_ROW)
+G_DEFINE_TYPE (CcSearchPanelRow, cc_search_panel_row, ADW_TYPE_ACTION_ROW)
 
 enum
 {
@@ -44,9 +44,11 @@ enum
 static guint signals[SIGNAL_LAST] = { 0, };
 
 static void
-move_up_button_clicked (GtkButton        *button,
-                        CcSearchPanelRow *self)
+move_up_cb (GSimpleAction *action,
+            GVariant      *parameter,
+            gpointer       user_data)
 {
+  CcSearchPanelRow *self = CC_SEARCH_PANEL_ROW (user_data);
   GtkListBox *list_box = GTK_LIST_BOX (gtk_widget_get_parent (GTK_WIDGET (self)));
   gint previous_idx = gtk_list_box_row_get_index (GTK_LIST_BOX_ROW (self)) - 1;
   GtkListBoxRow *previous_row = gtk_list_box_get_row_at_index (list_box, previous_idx);
@@ -61,9 +63,11 @@ move_up_button_clicked (GtkButton        *button,
 }
 
 static void
-move_down_button_clicked (GtkButton    *button,
-                          CcSearchPanelRow *self)
+move_down_cb (GSimpleAction *action,
+              GVariant      *parameter,
+              gpointer       user_data)
 {
+  CcSearchPanelRow *self = CC_SEARCH_PANEL_ROW (user_data);
   GtkListBox *list_box = GTK_LIST_BOX (gtk_widget_get_parent (GTK_WIDGET (self)));
   gint next_idx = gtk_list_box_row_get_index (GTK_LIST_BOX_ROW (self)) + 1;
   GtkListBoxRow *next_row = gtk_list_box_get_row_at_index (list_box, next_idx);
@@ -77,78 +81,67 @@ move_down_button_clicked (GtkButton    *button,
                  self);
 }
 
-static void
-drag_begin_cb (CcSearchPanelRow *self,
-               GdkDragContext   *drag_context)
+static GdkContentProvider *
+drag_prepare_cb (GtkDragSource    *source,
+                 double            x,
+                 double            y,
+                 CcSearchPanelRow *self)
 {
-  CcSearchPanelRow *drag_row;
+  self->drag_x = x;
+  self->drag_y = y;
+
+  return gdk_content_provider_new_typed (CC_TYPE_SEARCH_PANEL_ROW, self);
+}
+
+static void
+drag_begin_cb (GtkDragSource    *source,
+               GdkDrag          *drag,
+               CcSearchPanelRow *self)
+{
+  CcSearchPanelRow *panel_row;
   GtkAllocation alloc;
-  gint x = 0, y = 0;
+  GtkWidget *drag_icon;
 
   gtk_widget_get_allocation (GTK_WIDGET (self), &alloc);
 
-  gdk_window_get_device_position (gtk_widget_get_window (GTK_WIDGET (self)),
-                                  gdk_drag_context_get_device (drag_context),
-                                  &x, &y, NULL);
-
   self->drag_widget = GTK_LIST_BOX (gtk_list_box_new ());
-  gtk_widget_show (GTK_WIDGET (self->drag_widget));
   gtk_widget_set_size_request (GTK_WIDGET (self->drag_widget), alloc.width, alloc.height);
 
-  drag_row = cc_search_panel_row_new (self->app_info);
-  gtk_switch_set_active (drag_row->switcher, gtk_switch_get_active (self->switcher));
-  gtk_widget_show (GTK_WIDGET (drag_row));
-  gtk_container_add (GTK_CONTAINER (self->drag_widget), GTK_WIDGET (drag_row));
-  gtk_list_box_drag_highlight_row (self->drag_widget, GTK_LIST_BOX_ROW (drag_row));
+  panel_row = cc_search_panel_row_new (self->app_info);
+  gtk_switch_set_active (panel_row->switcher, gtk_switch_get_active (self->switcher));
+
+  gtk_list_box_append (GTK_LIST_BOX (self->drag_widget), GTK_WIDGET (panel_row));
+  gtk_list_box_drag_highlight_row (self->drag_widget, GTK_LIST_BOX_ROW (panel_row));
+
+  drag_icon = gtk_drag_icon_get_for_drag (drag);
+  gtk_drag_icon_set_child (GTK_DRAG_ICON (drag_icon), GTK_WIDGET (self->drag_widget));
+  gdk_drag_set_hotspot (drag, self->drag_x, self->drag_y);
 
-  gtk_drag_set_icon_widget (drag_context, GTK_WIDGET (self->drag_widget), x - alloc.x, y - alloc.y);
 }
 
-static void
-drag_end_cb (CcSearchPanelRow *self)
-{
-  g_clear_pointer ((GtkWidget **) &self->drag_widget, gtk_widget_destroy);
-}
-
-static void
-drag_data_get_cb (CcSearchPanelRow *self,
-                  GdkDragContext   *context,
-                  GtkSelectionData *selection_data,
-                  guint             info,
-                  guint             time_)
-{
-  gtk_selection_data_set (selection_data,
-                          gdk_atom_intern_static_string ("GTK_LIST_BOX_ROW"),
-                          32,
-                          (const guchar *)&self,
-                          sizeof (gpointer));
-}
-
-static void
-drag_data_received_cb (CcSearchPanelRow *self,
-                       GdkDragContext   *context,
-                       gint              x,
-                       gint              y,
-                       GtkSelectionData *selection_data,
-                       guint             info,
-                       guint             time_)
+static gboolean
+drop_cb (GtkDropTarget    *drop_target,
+         const GValue     *value,
+         gdouble           x,
+         gdouble           y,
+         CcSearchPanelRow *self)
 {
   CcSearchPanelRow *source;
 
-  source = *((CcSearchPanelRow **) gtk_selection_data_get_data (selection_data));
-  if (source == self)
-    return;
+  g_message ("Drop");
+
+  if (!G_VALUE_HOLDS (value, CC_TYPE_SEARCH_PANEL_ROW))
+    return FALSE;
+
+  source = g_value_get_object (value);
 
   g_signal_emit (source,
                  signals[SIGNAL_MOVE_ROW],
                  0,
                  self);
-}
 
-static GtkTargetEntry entries[] =
-{
-  { "GTK_LIST_BOX_ROW", GTK_TARGET_SAME_APP, 0 }
-};
+  return TRUE;
+}
 
 static void
 cc_search_panel_row_class_init (CcSearchPanelRowClass *klass)
@@ -158,18 +151,9 @@ cc_search_panel_row_class_init (CcSearchPanelRowClass *klass)
 
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/control-center/search/cc-search-panel-row.ui");
 
-  gtk_widget_class_bind_template_child (widget_class, CcSearchPanelRow, drag_handle);
   gtk_widget_class_bind_template_child (widget_class, CcSearchPanelRow, icon);
-  gtk_widget_class_bind_template_child (widget_class, CcSearchPanelRow, app_name);
   gtk_widget_class_bind_template_child (widget_class, CcSearchPanelRow, switcher);
 
-  gtk_widget_class_bind_template_callback (widget_class, drag_begin_cb);
-  gtk_widget_class_bind_template_callback (widget_class, drag_end_cb);
-  gtk_widget_class_bind_template_callback (widget_class, drag_data_get_cb);
-  gtk_widget_class_bind_template_callback (widget_class, drag_data_received_cb);
-  gtk_widget_class_bind_template_callback (widget_class, move_up_button_clicked);
-  gtk_widget_class_bind_template_callback (widget_class, move_down_button_clicked);
-
   signals[SIGNAL_MOVE_ROW] =
     g_signal_new ("move-row",
                   G_TYPE_FROM_CLASS (object_class),
@@ -181,13 +165,37 @@ cc_search_panel_row_class_init (CcSearchPanelRowClass *klass)
                   1, CC_TYPE_SEARCH_PANEL_ROW);
 }
 
+const GActionEntry row_entries[] = {
+  { "move-up", move_up_cb, NULL, NULL, NULL, { 0 }  },
+  { "move-down", move_down_cb, NULL, NULL, NULL, { 0 } }
+};
+
 static void
 cc_search_panel_row_init (CcSearchPanelRow *self)
 {
+  GtkDragSource *drag_source;
+  GtkDropTarget *drop_target;
+  GSimpleActionGroup *group;
+
   gtk_widget_init_template (GTK_WIDGET (self));
 
-  gtk_drag_source_set (GTK_WIDGET (self->drag_handle), GDK_BUTTON1_MASK, entries, 1, GDK_ACTION_MOVE);
-  gtk_drag_dest_set (GTK_WIDGET (self), GTK_DEST_DEFAULT_ALL, entries, 1, GDK_ACTION_MOVE);
+  drag_source = gtk_drag_source_new ();
+  gtk_drag_source_set_actions (drag_source, GDK_ACTION_MOVE);
+  g_signal_connect (drag_source, "prepare", G_CALLBACK (drag_prepare_cb), self);
+  g_signal_connect (drag_source, "drag-begin", G_CALLBACK (drag_begin_cb), self);
+  gtk_widget_add_controller (GTK_WIDGET (self), GTK_EVENT_CONTROLLER (drag_source));
+
+  drop_target = gtk_drop_target_new (CC_TYPE_SEARCH_PANEL_ROW, GDK_ACTION_MOVE);
+  gtk_drop_target_set_preload (drop_target, TRUE);
+  g_signal_connect (drop_target, "drop", G_CALLBACK (drop_cb), self);
+  gtk_widget_add_controller (GTK_WIDGET (self), GTK_EVENT_CONTROLLER (drop_target));
+
+  group = g_simple_action_group_new ();
+  g_action_map_add_action_entries (G_ACTION_MAP (group),
+                                   row_entries,
+                                   G_N_ELEMENTS (row_entries),
+                                   self);
+  gtk_widget_insert_action_group (GTK_WIDGET (self), "row", G_ACTION_GROUP (group));
 }
 
 CcSearchPanelRow *
@@ -195,7 +203,6 @@ cc_search_panel_row_new (GAppInfo *app_info)
 {
   CcSearchPanelRow *self;
   g_autoptr(GIcon) gicon = NULL;
-  gint width, height;
 
   self = g_object_new (CC_TYPE_SEARCH_PANEL_ROW, NULL);
   self->app_info = g_object_ref (app_info);
@@ -205,13 +212,10 @@ cc_search_panel_row_new (GAppInfo *app_info)
     gicon = g_themed_icon_new ("application-x-executable");
   else
     g_object_ref (gicon);
-  gtk_image_set_from_gicon (self->icon, gicon, GTK_ICON_SIZE_DND);
-  gtk_icon_size_lookup (GTK_ICON_SIZE_DND, &width, &height);
-  gtk_image_set_pixel_size (self->icon, MAX (width, height));
+  gtk_image_set_from_gicon (self->icon, gicon);
 
-  gtk_label_set_text (self->app_name, g_app_info_get_name (app_info));
-
-  gtk_drag_source_set (GTK_WIDGET (self->drag_handle), GDK_BUTTON1_MASK, entries, 1, GDK_ACTION_MOVE);
+  adw_preferences_row_set_title (ADW_PREFERENCES_ROW (self),
+                                 g_app_info_get_name (app_info));
 
   return self;
 }
