@@ -29,6 +29,7 @@
 #include "shell/cc-debug.h"
 #include "cc-wacom-panel.h"
 #include "cc-wacom-page.h"
+#include "cc-wacom-ekr-page.h"
 #include "cc-wacom-stylus-page.h"
 #include "cc-wacom-resources.h"
 #include "cc-drawing-area.h"
@@ -36,31 +37,40 @@
 #include "gsd-device-manager.h"
 
 #ifdef GDK_WINDOWING_WAYLAND
-#include <gdk/gdkwayland.h>
+#include <gdk/wayland/gdkwayland.h>
 #endif
 
-#define WID(x) (GtkWidget *) gtk_builder_get_object (self->builder, x)
+#define EKR_VENDOR "056a"
+#define EKR_PRODUCT "0331"
 
 struct _CcWacomPanel
 {
 	CcPanel           parent_instance;
 
-	GtkBuilder       *builder;
-	GtkWidget        *stack;
-	GtkWidget        *switcher;
-	GtkWidget        *tablet_notebook;
-	GtkWidget        *stylus_notebook;
 	GtkWidget        *test_popover;
 	GtkWidget        *test_draw_area;
 	GtkWidget        *test_button;
+	GtkWidget        *scrollable;
+	GtkWidget        *tablets;
+	GtkWidget        *styli;
+	GtkWidget        *initial_state_stack;
+	GtkWidget        *panel_view;
+	GtkWidget        *panel_empty_state;
 	GHashTable       *devices; /* key=GsdDevice, value=CcWacomDevice */
 	GHashTable       *pages; /* key=CcWacomDevice, value=GtkWidget */
 	GHashTable       *stylus_pages; /* key=CcWacomTool, value=GtkWidget */
+	guint             mock_stylus_id;
 
 	CcTabletToolMap  *tablet_tool_map;
 
+	GtkAdjustment    *vadjustment;
+	GtkGesture       *stylus_gesture;
+
+	GtkWidget        *highlighted_widget;
+
 	/* DBus */
 	GDBusProxy    *proxy;
+	GDBusProxy    *input_mapping_proxy;
 };
 
 CC_PANEL_REGISTER (CcWacomPanel, cc_wacom_panel)
@@ -132,12 +142,27 @@ lookup_wacom_device (CcWacomPanel *self,
 	return NULL;
 }
 
+static void
+highlight_widget (CcWacomPanel *self, GtkWidget *widget)
+{
+	double y;
+
+	if (self->highlighted_widget == widget)
+		return;
+
+	gtk_widget_translate_coordinates (widget,
+					  self->scrollable,
+					  0, 0,
+					  NULL, &y);
+	gtk_adjustment_set_value (self->vadjustment, y);
+	self->highlighted_widget = widget;
+}
+
 static CcWacomPage *
-set_device_page (CcWacomPanel *self, const gchar *device_name)
+update_highlighted_device (CcWacomPanel *self, const gchar *device_name)
 {
 	CcWacomPage *page;
 	CcWacomDevice *wacom_device;
-	gint current;
 
 	if (device_name == NULL)
 		return NULL;
@@ -149,8 +174,7 @@ set_device_page (CcWacomPanel *self, const gchar *device_name)
 	}
 
 	page = g_hash_table_lookup (self->pages, wacom_device);
-	current = gtk_notebook_page_num (GTK_NOTEBOOK (self->tablet_notebook), GTK_WIDGET (page));
-	gtk_notebook_set_current_page (GTK_NOTEBOOK (self->tablet_notebook), current);
+	highlight_widget (self, GTK_WIDGET (page));
 
 	return page;
 }
@@ -178,7 +202,7 @@ run_operation_from_params (CcWacomPanel *self, GVariant *parameters)
 
 	switch (n_params) {
 		case 3:
-			page = set_device_page (self, device_name);
+			page = update_highlighted_device (self, device_name);
 			if (page == NULL)
 				return;
 
@@ -199,7 +223,7 @@ run_operation_from_params (CcWacomPanel *self, GVariant *parameters)
 				g_warning ("Ignoring unrecognized operation '%s'", operation);
 			}
 		case 2:
-			set_device_page (self, device_name);
+			update_highlighted_device (self, device_name);
 			break;
 		case 1:
 			g_assert_not_reached ();
@@ -254,13 +278,21 @@ static void
 cc_wacom_panel_dispose (GObject *object)
 {
 	CcWacomPanel *self = CC_WACOM_PANEL (object);
+	CcShell *shell;
+
+	shell = cc_panel_get_shell (CC_PANEL (self));
+	if (shell) {
+		gtk_widget_remove_controller (GTK_WIDGET (shell),
+					      GTK_EVENT_CONTROLLER (self->stylus_gesture));
+	}
 
-	g_clear_object (&self->builder);
 
 	g_clear_pointer (&self->devices, g_hash_table_unref);
 	g_clear_object (&self->proxy);
+	g_clear_object (&self->input_mapping_proxy);
 	g_clear_pointer (&self->pages, g_hash_table_unref);
 	g_clear_pointer (&self->stylus_pages, g_hash_table_unref);
+	g_clear_handle_id (&self->mock_stylus_id, g_source_remove);
 
 	G_OBJECT_CLASS (cc_wacom_panel_parent_class)->dispose (object);
 }
@@ -289,7 +321,8 @@ check_remove_stylus_pages (CcWacomPanel *self)
 	while (g_hash_table_iter_next (&iter, (gpointer*) &tool, (gpointer*) &page)) {
 		if (g_list_find (total, tool))
 			continue;
-		gtk_widget_destroy (page);
+
+		gtk_box_remove (GTK_BOX (self->styli), page);
 		g_hash_table_iter_remove (&iter);
 	}
 }
@@ -304,15 +337,9 @@ add_stylus (CcWacomPanel *self,
 		return FALSE;
 
 	page = cc_wacom_stylus_page_new (tool);
-	cc_wacom_stylus_page_set_navigation (CC_WACOM_STYLUS_PAGE (page),
-					     GTK_NOTEBOOK (self->stylus_notebook));
-	gtk_widget_show (page);
-	gtk_notebook_append_page (GTK_NOTEBOOK (self->stylus_notebook), page, NULL);
+	gtk_box_append (GTK_BOX (self->styli), page);
 	g_hash_table_insert (self->stylus_pages, tool, page);
 
-	if (gtk_notebook_get_current_page (GTK_NOTEBOOK (self->stylus_notebook)) == 0)
-		gtk_notebook_set_current_page (GTK_NOTEBOOK (self->stylus_notebook), 1);
-
 	return TRUE;
 }
 
@@ -323,13 +350,32 @@ update_test_button (CcWacomPanel *self)
 		return;
 
 	if (g_hash_table_size (self->devices) == 0) {
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->test_button), FALSE);
+		gtk_popover_popdown (GTK_POPOVER (self->test_popover));
 		gtk_widget_set_sensitive (self->test_button, FALSE);
 	} else {
 		gtk_widget_set_sensitive (self->test_button, TRUE);
 	}
 }
 
+static void
+update_initial_state (CcWacomPanel *self)
+{
+	gtk_stack_set_visible_child (GTK_STACK (self->initial_state_stack),
+				     g_hash_table_size (self->devices) == 0 ?
+				     self->panel_empty_state :
+				     self->panel_view);
+}
+
+static void
+update_highlighted_stylus (CcWacomPanel *panel,
+			   CcWacomTool  *stylus)
+{
+	GtkWidget *widget;
+
+	widget = g_hash_table_lookup (panel->stylus_pages, stylus);
+	highlight_widget (panel, widget);
+}
+
 static void
 update_current_tool (CcWacomPanel  *panel,
 		     GdkDevice     *device,
@@ -340,7 +386,6 @@ update_current_tool (CcWacomPanel  *panel,
 	CcWacomTool *stylus;
 	GsdDevice *gsd_device;
 	guint64 serial, id;
-	gboolean added;
 
 	if (!tool)
 		return;
@@ -390,47 +435,60 @@ update_current_tool (CcWacomPanel  *panel,
 			return;
         }
 
-	added = add_stylus (panel, stylus);
+	add_stylus (panel, stylus);
 
-	if (added) {
-		if (panel->stylus_notebook ==
-		    gtk_stack_get_visible_child (GTK_STACK (panel->stack))) {
-			GtkWidget *widget;
-			gint page;
-
-			widget = g_hash_table_lookup (panel->stylus_pages, stylus);
-			page = gtk_notebook_page_num (GTK_NOTEBOOK (panel->stylus_notebook), widget);
-			gtk_notebook_set_current_page (GTK_NOTEBOOK (panel->stylus_notebook), page);
-		} else {
-			gtk_container_child_set (GTK_CONTAINER (panel->stack),
-						 panel->stylus_notebook,
-						 "needs-attention", TRUE,
-						 NULL);
-		}
-	}
+	update_highlighted_stylus (panel, stylus);
 
 	cc_tablet_tool_map_add_relation (panel->tablet_tool_map,
 					 wacom_device, stylus);
 }
 
-static gboolean
-on_shell_event_cb (CcWacomPanel *panel,
-		   GdkEvent     *event)
+static void
+on_stylus_proximity_cb (GtkGestureStylus *gesture,
+			double            x,
+			double            y,
+			CcWacomPanel     *panel)
 {
-	if (event->type == GDK_MOTION_NOTIFY) {
-		update_current_tool (panel,
-				     gdk_event_get_source_device (event),
-				     gdk_event_get_device_tool (event));
+	GdkDevice *device;
+	GdkDeviceTool *tool;
+
+	device = gtk_event_controller_get_current_event_device (GTK_EVENT_CONTROLLER (gesture));
+	tool = gtk_gesture_stylus_get_device_tool (gesture);
+	update_current_tool (panel, device, tool);
+}
+
+static gboolean
+show_mock_stylus_cb (gpointer user_data)
+{
+	CcWacomPanel *panel = user_data;
+	GList *device_list;
+	CcWacomDevice *wacom_device;
+	CcWacomTool *stylus;
+
+	panel->mock_stylus_id = 0;
+
+	device_list = g_hash_table_get_values (panel->devices);
+	if (device_list == NULL) {
+		g_warning ("Could not create fake stylus event because could not find tablet device");
+		return G_SOURCE_REMOVE;
 	}
 
-	return GDK_EVENT_PROPAGATE;
+	wacom_device = device_list->data;
+	g_list_free (device_list);
+
+	stylus = cc_wacom_tool_new (0, 0, wacom_device);
+	add_stylus (panel, stylus);
+	update_highlighted_stylus (panel, stylus);
+	cc_tablet_tool_map_add_relation (panel->tablet_tool_map,
+					 wacom_device, stylus);
+
+	return G_SOURCE_REMOVE;
 }
 
 static void
 cc_wacom_panel_constructed (GObject *object)
 {
 	CcWacomPanel *self = CC_WACOM_PANEL (object);
-	GtkWidget *button;
 	CcShell *shell;
 
 	G_OBJECT_CLASS (cc_wacom_panel_parent_class)->constructed (object);
@@ -438,32 +496,14 @@ cc_wacom_panel_constructed (GObject *object)
 	/* Add test area button to shell header. */
 	shell = cc_panel_get_shell (CC_PANEL (self));
 
-	button = gtk_toggle_button_new_with_mnemonic (_("Test Your _Settings"));
-	gtk_style_context_add_class (gtk_widget_get_style_context (button),
-				     "text-button");
-	gtk_widget_set_valign (button, GTK_ALIGN_CENTER);
-	gtk_widget_set_visible (button, TRUE);
+	self->stylus_gesture = gtk_gesture_stylus_new ();
+	g_signal_connect (self->stylus_gesture, "proximity",
+			  G_CALLBACK (on_stylus_proximity_cb), self);
+	gtk_widget_add_controller (GTK_WIDGET (shell),
+				   GTK_EVENT_CONTROLLER (self->stylus_gesture));
 
-	cc_shell_embed_widget_in_header (shell, button, GTK_POS_RIGHT);
-
-	self->test_popover = gtk_popover_new (button);
-	gtk_container_set_border_width (GTK_CONTAINER (self->test_popover), 6);
-
-	self->test_draw_area = cc_drawing_area_new ();
-	gtk_widget_set_size_request (self->test_draw_area, 400, 300);
-	gtk_container_add (GTK_CONTAINER (self->test_popover),
-			   self->test_draw_area);
-	gtk_widget_show (self->test_draw_area);
-
-	g_object_bind_property (button, "active",
-				self->test_popover, "visible",
-				G_BINDING_BIDIRECTIONAL);
-
-	g_signal_connect_object (shell, "event",
-				 G_CALLBACK (on_shell_event_cb), self, G_CONNECT_SWAPPED);
-
-	self->test_button = button;
-	update_test_button (self);
+	if (g_getenv ("UMOCKDEV_DIR") != NULL)
+		self->mock_stylus_id = g_idle_add (show_mock_stylus_cb, self);
 }
 
 static const char *
@@ -472,19 +512,12 @@ cc_wacom_panel_get_help_uri (CcPanel *panel)
   return "help:gnome-help/wacom";
 }
 
-static GtkWidget *
-cc_wacom_panel_get_title_widget (CcPanel *panel)
-{
-	CcWacomPanel *self = CC_WACOM_PANEL (panel);
-
-	return self->switcher;
-}
-
 static void
 cc_wacom_panel_class_init (CcWacomPanelClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	CcPanelClass *panel_class = CC_PANEL_CLASS (klass);
+	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
 	object_class->get_property = cc_wacom_panel_get_property;
 	object_class->set_property = cc_wacom_panel_set_property;
@@ -492,21 +525,23 @@ cc_wacom_panel_class_init (CcWacomPanelClass *klass)
 	object_class->constructed = cc_wacom_panel_constructed;
 
 	panel_class->get_help_uri = cc_wacom_panel_get_help_uri;
-	panel_class->get_title_widget = cc_wacom_panel_get_title_widget;
 
 	g_object_class_override_property (object_class, PROP_PARAMETERS, "parameters");
-}
 
-static void
-update_current_page (CcWacomPanel  *self)
-{
-	int num_pages;
+	g_type_ensure (CC_TYPE_DRAWING_AREA);
 
-	num_pages = gtk_notebook_get_n_pages (GTK_NOTEBOOK (self->tablet_notebook));
-	if (num_pages > 1)
-		gtk_notebook_set_current_page (GTK_NOTEBOOK (self->tablet_notebook), 1);
+	gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/control-center/wacom/cc-wacom-panel.ui");
 
-	update_test_button (self);
+	gtk_widget_class_bind_template_child (widget_class, CcWacomPanel, scrollable);
+	gtk_widget_class_bind_template_child (widget_class, CcWacomPanel, test_button);
+	gtk_widget_class_bind_template_child (widget_class, CcWacomPanel, test_popover);
+	gtk_widget_class_bind_template_child (widget_class, CcWacomPanel, test_draw_area);
+	gtk_widget_class_bind_template_child (widget_class, CcWacomPanel, tablets);
+	gtk_widget_class_bind_template_child (widget_class, CcWacomPanel, styli);
+	gtk_widget_class_bind_template_child (widget_class, CcWacomPanel, initial_state_stack);
+	gtk_widget_class_bind_template_child (widget_class, CcWacomPanel, panel_empty_state);
+	gtk_widget_class_bind_template_child (widget_class, CcWacomPanel, panel_view);
+	gtk_widget_class_bind_template_child (widget_class, CcWacomPanel, vadjustment);
 }
 
 static void
@@ -517,6 +552,7 @@ add_known_device (CcWacomPanel *self,
 	GsdDeviceType device_type;
 	g_autoptr(GList) tools = NULL;
 	GtkWidget *page;
+	gboolean is_ekr = FALSE;
 	GList *l;
 
 	device_type = gsd_device_get_device_type (gsd_device);
@@ -525,12 +561,27 @@ add_known_device (CcWacomPanel *self,
 		return;
 
 	if ((device_type &
-	     (GSD_DEVICE_TYPE_PAD |
-	      GSD_DEVICE_TYPE_TOUCHSCREEN |
+	     (GSD_DEVICE_TYPE_TOUCHSCREEN |
 	      GSD_DEVICE_TYPE_TOUCHPAD)) != 0) {
 		return;
 	}
 
+	if ((device_type & GSD_DEVICE_TYPE_PAD) != 0) {
+		const char *vendor, *product;
+
+		gsd_device_get_device_ids (gsd_device, &vendor, &product);
+		is_ekr = (g_strcmp0 (vendor, EKR_VENDOR) == 0 &&
+			  g_strcmp0 (product, EKR_PRODUCT) == 0);
+
+		/* Express key remote is an special case, as it is an
+		 * external pad device, we want to distinctly show it
+		 * in the list. Other pads are mounted on a tablet, which
+		 * get their own entries.
+		 */
+		if (!is_ekr)
+			return;
+	}
+
 	device = cc_wacom_device_new (gsd_device);
 	if (!device)
 		return;
@@ -543,10 +594,12 @@ add_known_device (CcWacomPanel *self,
 		add_stylus (self, l->data);
 	}
 
-	page = cc_wacom_page_new (self, device);
-	cc_wacom_page_set_navigation (CC_WACOM_PAGE (page), GTK_NOTEBOOK (self->tablet_notebook), TRUE);
-	gtk_widget_show (page);
-	gtk_notebook_append_page (GTK_NOTEBOOK (self->tablet_notebook), page, NULL);
+	if (is_ekr)
+		page = cc_wacom_ekr_page_new (self, device);
+	else
+		page = cc_wacom_page_new (self, device);
+
+	gtk_box_append (GTK_BOX (self->tablets), page);
 	g_hash_table_insert (self->pages, device, page);
 }
 
@@ -564,12 +617,13 @@ device_removed_cb (CcWacomPanel     *self,
 	page = g_hash_table_lookup (self->pages, device);
 	if (page) {
 		g_hash_table_remove (self->pages, device);
-		gtk_widget_destroy (page);
+		gtk_box_remove (GTK_BOX (self->tablets), page);
 	}
 
 	g_hash_table_remove (self->devices, gsd_device);
 	check_remove_stylus_pages (self);
-	update_current_page (self);
+	update_test_button (self);
+	update_initial_state (self);
 }
 
 static void
@@ -577,14 +631,8 @@ device_added_cb (CcWacomPanel *self,
 		 GsdDevice    *device)
 {
 	add_known_device (self, device);
-	update_current_page (self);
-}
-
-static gboolean
-link_activated (CcWacomPanel  *self)
-{
-	cc_wacom_panel_switch_to_panel (self, "bluetooth");
-	return TRUE;
+	update_test_button (self);
+	update_initial_state (self);
 }
 
 void
@@ -619,61 +667,33 @@ got_osd_proxy_cb (GObject      *source_object,
 }
 
 static void
-enbiggen_label (GtkLabel *label)
+got_input_mapping_proxy_cb (GObject      *source_object,
+			    GAsyncResult *res,
+			    gpointer      data)
 {
-	const char *str;
-	g_autofree char *new_str = NULL;
+	g_autoptr(GError) error = NULL;
+	CcWacomPanel *self;
 
-	str = gtk_label_get_text (label);
-	new_str = g_strdup_printf ("<big>%s</big>", str);
-	gtk_label_set_markup (label, new_str);
-}
+	self = CC_WACOM_PANEL (data);
+	self->input_mapping_proxy = g_dbus_proxy_new_for_bus_finish (res, &error);
 
-static void
-on_stack_visible_child_notify_cb (CcWacomPanel *panel)
-{
-	GtkWidget *child;
-
-	child = gtk_stack_get_visible_child (GTK_STACK (panel->stack));
-
-	if (child == panel->stylus_notebook) {
-		gtk_container_child_set (GTK_CONTAINER (panel->stack),
-					 panel->stylus_notebook,
-					 "needs-attention", FALSE,
-					 NULL);
+	if (self->input_mapping_proxy == NULL) {
+		g_printerr ("Error creating input mapping proxy: %s\n", error->message);
+		return;
 	}
 }
 
 static void
 cc_wacom_panel_init (CcWacomPanel *self)
 {
-	GtkWidget *widget;
 	GsdDeviceManager *device_manager;
 	g_autoptr(GList) devices = NULL;
 	GList *l;
 	g_autoptr(GError) error = NULL;
-	char *objects[] = {
-		"main-box",
-		"no-stylus-page",
-		NULL
-	};
 
         g_resources_register (cc_wacom_get_resource ());
 
-	self->builder = gtk_builder_new ();
-
-	gtk_builder_add_objects_from_resource (self->builder,
-                                               "/org/gnome/control-center/wacom/gnome-wacom-properties.ui",
-                                               objects,
-                                               &error);
-	gtk_builder_add_objects_from_resource (self->builder,
-                                               "/org/gnome/control-center/wacom/wacom-stylus-page.ui",
-                                               objects,
-                                               &error);
-	if (error != NULL) {
-		g_warning ("Error loading UI file: %s", error->message);
-		return;
-	}
+	gtk_widget_init_template (GTK_WIDGET (self));
 
 	self->tablet_tool_map = cc_tablet_tool_map_new ();
 
@@ -687,58 +707,15 @@ cc_wacom_panel_init (CcWacomPanel *self)
 				  got_osd_proxy_cb,
 				  self);
 
-	/* Stack + Switcher */
-	self->stack = gtk_stack_new ();
-	g_object_set (G_OBJECT (self->stack),
-		      "margin-top", 30,
-		      "margin-end", 30,
-		      "margin-start", 30,
-		      "margin-bottom", 30,
-		      NULL);
-
-	g_signal_connect_object (self->stack, "notify::visible-child",
-	                         G_CALLBACK (on_stack_visible_child_notify_cb), self, G_CONNECT_SWAPPED);
-
-	self->switcher = gtk_stack_switcher_new ();
-	gtk_stack_switcher_set_stack (GTK_STACK_SWITCHER (self->switcher),
-				      GTK_STACK (self->stack));
-	gtk_widget_show (self->switcher);
-
-	gtk_container_add (GTK_CONTAINER (self), GTK_WIDGET (self->stack));
-	gtk_widget_show (self->stack);
-
-	self->tablet_notebook = gtk_notebook_new ();
-	gtk_widget_show (self->tablet_notebook);
-	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (self->tablet_notebook), FALSE);
-	gtk_notebook_set_show_border (GTK_NOTEBOOK (self->tablet_notebook), FALSE);
-	gtk_widget_set_vexpand (self->tablet_notebook, TRUE);
-
-	self->stylus_notebook = gtk_notebook_new ();
-	gtk_widget_show (self->stylus_notebook);
-	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (self->stylus_notebook), FALSE);
-	gtk_notebook_set_show_border (GTK_NOTEBOOK (self->stylus_notebook), FALSE);
-	gtk_container_set_border_width (GTK_CONTAINER (self->stylus_notebook), 0);
-	gtk_widget_set_vexpand (self->stylus_notebook, TRUE);
-
-	gtk_stack_add_titled (GTK_STACK (self->stack),
-			      self->stylus_notebook, "stylus",
-			      _("Stylus"));
-	gtk_stack_add_titled (GTK_STACK (self->stack),
-			      self->tablet_notebook, "tablet",
-			      _("Tablet"));
-
-	/* No styli page */
-	widget = WID ("no-stylus-page");
-	enbiggen_label (GTK_LABEL (WID ("no-stylus-label1")));
-	gtk_notebook_append_page (GTK_NOTEBOOK (self->stylus_notebook), widget, NULL);
-
-	/* No tablets page */
-	widget = WID ("main-box");
-	enbiggen_label (GTK_LABEL (WID ("advice-label1")));
-	gtk_notebook_append_page (GTK_NOTEBOOK (self->tablet_notebook), widget, NULL);
-
-	g_signal_connect_object (WID ("linkbutton"), "activate-link",
-			         G_CALLBACK (link_activated), self, G_CONNECT_SWAPPED);
+	g_dbus_proxy_new_for_bus (G_BUS_TYPE_SESSION,
+				  G_DBUS_PROXY_FLAGS_NONE,
+				  NULL,
+				  "org.gnome.Shell",
+				  "/org/gnome/Mutter/InputMapping",
+				  "org.gnome.Mutter.InputMapping",
+				  cc_panel_get_cancellable (CC_PANEL (self)),
+				  got_input_mapping_proxy_cb,
+				  self);
 
 	self->devices = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL, g_object_unref);
 	self->pages = g_hash_table_new (NULL, NULL);
@@ -755,7 +732,8 @@ cc_wacom_panel_init (CcWacomPanel *self)
 	for (l = devices; l ; l = l->next)
 		add_known_device (self, l->data);
 
-	update_current_page (self);
+	update_test_button (self);
+	update_initial_state (self);
 }
 
 GDBusProxy *
@@ -765,3 +743,11 @@ cc_wacom_panel_get_gsd_wacom_bus_proxy (CcWacomPanel *self)
 
 	return self->proxy;
 }
+
+GDBusProxy *
+cc_wacom_panel_get_input_mapping_bus_proxy (CcWacomPanel *self)
+{
+	g_return_val_if_fail (CC_IS_WACOM_PANEL (self), NULL);
+
+	return self->input_mapping_proxy;
+}
