@@ -32,7 +32,4 @@ GtkWidget * cc_wacom_stylus_page_new (CcWacomTool *stylus);
 
 CcWacomTool * cc_wacom_stylus_page_get_tool (CcWacomStylusPage *page);
 
-void cc_wacom_stylus_page_set_navigation (CcWacomStylusPage *page,
-					  GtkNotebook *notebook);
-
 G_END_DECLS
