@@ -19,14 +19,18 @@
 
 #pragma once
 
-#include <gtk/gtk.h>
+#include "cc-shell-model.h"
+
+#include <adwaita.h>
 
 G_BEGIN_DECLS
 
 #define CC_TYPE_APPLICATION (cc_application_get_type())
 
-G_DECLARE_FINAL_TYPE (CcApplication, cc_application, CC, APPLICATION, GtkApplication)
+G_DECLARE_FINAL_TYPE (CcApplication, cc_application, CC, APPLICATION, AdwApplication)
 
 GtkApplication        *cc_application_new                    (void);
 
-G_END_DECLS
\ No newline at end of file
+CcShellModel          *cc_application_get_model              (CcApplication *self);
+
+G_END_DECLS
