@@ -49,7 +49,7 @@ extern GType cc_network_panel_get_type (void);
 extern GType cc_wifi_panel_get_type (void);
 #endif /* BUILD_NETWORK */
 extern GType cc_notifications_panel_get_type (void);
-extern GType cc_goa_panel_get_type (void);
+extern GType cc_online_accounts_panel_get_type (void);
 extern GType cc_power_panel_get_type (void);
 extern GType cc_printers_panel_get_type (void);
 extern GType cc_region_panel_get_type (void);
@@ -69,10 +69,11 @@ extern GType cc_wacom_panel_get_type (void);
 extern GType cc_wwan_panel_get_type (void);
 #endif /* BUILD_WWAN */
 extern GType cc_location_panel_get_type (void);
+extern GType cc_firmware_security_panel_get_type (void);
 extern GType cc_camera_panel_get_type (void);
 extern GType cc_microphone_panel_get_type (void);
 extern GType cc_usage_panel_get_type (void);
-extern GType cc_lock_panel_get_type (void);
+extern GType cc_screen_panel_get_type (void);
 extern GType cc_diagnostics_panel_get_type (void);
 
 /* Static init functions */
@@ -86,6 +87,7 @@ extern void cc_wacom_panel_static_init_func (void);
 #ifdef BUILD_WWAN
 extern void cc_wwan_panel_static_init_func (void);
 #endif /* BUILD_WWAN */
+extern void cc_firmware_security_panel_static_init_func (void);
 
 #define PANEL_TYPE(name, get_type, init_func) { name, get_type, init_func }
 
@@ -111,7 +113,7 @@ static CcPanelLoaderVtable default_panels[] =
   PANEL_TYPE("info-overview",    cc_info_overview_panel_get_type,        NULL),
   PANEL_TYPE("keyboard",         cc_keyboard_panel_get_type,             NULL),
   PANEL_TYPE("location",         cc_location_panel_get_type,             NULL),
-  PANEL_TYPE("lock",             cc_lock_panel_get_type,                 NULL),
+  PANEL_TYPE("firmware-security",cc_firmware_security_panel_get_type,    cc_firmware_security_panel_static_init_func),
   PANEL_TYPE("microphone",       cc_microphone_panel_get_type,           NULL),
   PANEL_TYPE("mouse",            cc_mouse_panel_get_type,                NULL),
   PANEL_TYPE("multitasking",     cc_multitasking_panel_get_type,         NULL),
@@ -120,11 +122,12 @@ static CcPanelLoaderVtable default_panels[] =
   PANEL_TYPE("wifi",             cc_wifi_panel_get_type,                 cc_wifi_panel_static_init_func),
 #endif
   PANEL_TYPE("notifications",    cc_notifications_panel_get_type,        NULL),
-  PANEL_TYPE("online-accounts",  cc_goa_panel_get_type,                  NULL),
+  PANEL_TYPE("online-accounts",  cc_online_accounts_panel_get_type,      NULL),
   PANEL_TYPE("power",            cc_power_panel_get_type,                NULL),
   PANEL_TYPE("printers",         cc_printers_panel_get_type,             NULL),
   PANEL_TYPE("region",           cc_region_panel_get_type,               NULL),
   PANEL_TYPE("removable-media",  cc_removable_media_panel_get_type,      NULL),
+  PANEL_TYPE("screen",           cc_screen_panel_get_type,               NULL),
   PANEL_TYPE("search",           cc_search_panel_get_type,               NULL),
   PANEL_TYPE("sharing",          cc_sharing_panel_get_type,              NULL),
   PANEL_TYPE("sound",            cc_sound_panel_get_type,                NULL),
@@ -218,6 +221,7 @@ ensure_panel_types (void)
 CcPanel *
 cc_panel_loader_load_by_name (CcShell     *shell,
                               const gchar *name,
+                              const gchar *title,
                               GVariant    *parameters)
 {
   GType (*get_type) (void);
@@ -230,6 +234,7 @@ cc_panel_loader_load_by_name (CcShell     *shell,
   return g_object_new (get_type (),
                        "shell", shell,
                        "parameters", parameters,
+                       "title", title,
                        NULL);
 }
 
