@@ -31,14 +31,11 @@
 #include <gio/gio.h>
 #include <gio/gdesktopappinfo.h>
 #include <gtk/gtk.h>
-#include <gdk/gdkkeysyms.h>
-#include <gdk/gdkx.h>
-#include <handy.h>
 #include <string.h>
 #include <time.h>
 
 #include "cc-application.h"
-#include "cc-panel.h"
+#include "cc-panel-private.h"
 #include "cc-shell.h"
 #include "cc-shell-model.h"
 #include "cc-panel-list.h"
@@ -51,99 +48,81 @@
 
 struct _CcWindow
 {
-  GtkApplicationWindow parent;
+  AdwApplicationWindow parent;
 
-  GtkRevealer       *back_revealer;
   GtkMessageDialog  *development_warning_dialog;
-  GtkHeaderBar      *header;
-  HdyLeaflet        *header_box;
-  GtkSizeGroup      *header_sizegroup;
-  HdyLeaflet        *main_leaflet;
-  GtkHeaderBar      *panel_headerbar;
+  AdwHeaderBar      *header;
+  AdwLeaflet        *main_leaflet;
   CcPanelList       *panel_list;
   GtkButton         *previous_button;
   GtkSearchBar      *search_bar;
   GtkToggleButton   *search_button;
   GtkSearchEntry    *search_entry;
   GtkBox            *sidebar_box;
+  AdwWindowTitle    *sidebar_title_widget;
   GtkStack          *stack;
-  GtkBox            *top_left_box;
-  GtkBox            *top_right_box;
 
+  GtkWidget  *old_panel;
   GtkWidget  *current_panel;
   char       *current_panel_id;
   GQueue     *previous_panels;
 
-  GPtrArray  *custom_widgets;
+  GtkWidget  *custom_titlebar;
 
   CcShellModel *store;
 
   CcPanel *active_panel;
   GSettings *settings;
 
+  gboolean folded;
+
   CcPanelListView previous_list_view;
 };
 
 static void     cc_shell_iface_init         (CcShellInterface      *iface);
 
-G_DEFINE_TYPE_WITH_CODE (CcWindow, cc_window, GTK_TYPE_APPLICATION_WINDOW,
+G_DEFINE_TYPE_WITH_CODE (CcWindow, cc_window, ADW_TYPE_APPLICATION_WINDOW,
                          G_IMPLEMENT_INTERFACE (CC_TYPE_SHELL, cc_shell_iface_init))
 
 enum
 {
   PROP_0,
   PROP_ACTIVE_PANEL,
-  PROP_MODEL
+  PROP_MODEL,
+  PROP_FOLDED,
 };
 
 /* Auxiliary methods */
+static void
+load_window_state (CcWindow *self)
+{
+  gint current_width = -1;
+  gint current_height = -1;
+  gboolean maximized = FALSE;
+
+  g_settings_get (self->settings,
+                 "window-state",
+                 "(iib)",
+                 &current_width,
+                 &current_height,
+                 &maximized);
+
+  if (current_width != -1 && current_height != -1)
+    gtk_window_set_default_size (GTK_WINDOW (self), current_width, current_height);
+  if (maximized)
+    gtk_window_maximize (GTK_WINDOW (self));
+}
+
 static gboolean
 in_flatpak_sandbox (void)
 {
   return g_strcmp0 (PROFILE, "development") == 0;
 }
 
-static void
-remove_all_custom_widgets (CcWindow *self)
-{
-  GtkWidget *parent;
-  GtkWidget *widget;
-  guint i;
-
-  CC_ENTRY;
-
-  /* remove from the header */
-  for (i = 0; i < self->custom_widgets->len; i++)
-    {
-      widget = g_ptr_array_index (self->custom_widgets, i);
-      parent = gtk_widget_get_parent (widget);
-
-      g_assert (parent == GTK_WIDGET (self->top_right_box) || parent == GTK_WIDGET (self->top_left_box));
-      gtk_container_remove (GTK_CONTAINER (parent), widget);
-    }
-  g_ptr_array_set_size (self->custom_widgets, 0);
-
-  CC_EXIT;
-}
-
-static void
-show_panel (CcWindow *self)
-{
-  hdy_leaflet_set_visible_child (self->main_leaflet, GTK_WIDGET (self->stack));
-  hdy_leaflet_set_visible_child (self->header_box, GTK_WIDGET (self->panel_headerbar));
-}
-
-static void
-show_sidebar (CcWindow *self)
-{
-  hdy_leaflet_set_visible_child (self->main_leaflet, GTK_WIDGET (self->sidebar_box));
-  hdy_leaflet_set_visible_child (self->header_box, GTK_WIDGET (self->header));
-}
-
 static void
 on_sidebar_activated_cb (CcWindow *self)
 {
-  show_panel (self);
+  adw_leaflet_navigate (self->main_leaflet, ADW_NAVIGATION_DIRECTION_FORWARD);
 }
 
 static gboolean
@@ -156,7 +135,6 @@ activate_panel (CcWindow          *self,
 {
   g_autoptr(GTimer) timer = NULL;
   GtkWidget *sidebar_widget;
-  GtkWidget *title_widget;
   gdouble ellapsed_time;
 
   CC_ENTRY;
@@ -167,9 +145,6 @@ activate_panel (CcWindow          *self,
   if (visibility == CC_PANEL_HIDDEN)
     CC_RETURN (FALSE);
 
-  /* clear any custom widgets */
-  remove_all_custom_widgets (self);
-
   timer = g_timer_new ();
 
   g_settings_set_string (self->settings, "last-panel", id);
@@ -179,23 +154,15 @@ activate_panel (CcWindow          *self,
 
   if (self->current_panel)
     g_signal_handlers_disconnect_by_data (self->current_panel, self);
-  self->current_panel = GTK_WIDGET (cc_panel_loader_load_by_name (CC_SHELL (self), id, parameters));
+  self->current_panel = GTK_WIDGET (cc_panel_loader_load_by_name (CC_SHELL (self), id, name, parameters));
+  cc_panel_set_folded (CC_PANEL (self->current_panel), adw_leaflet_get_folded (self->main_leaflet));
   cc_shell_set_active_panel (CC_SHELL (self), CC_PANEL (self->current_panel));
-  gtk_widget_show (self->current_panel);
 
   gtk_stack_add_named (self->stack, self->current_panel, id);
 
   /* switch to the new panel */
-  gtk_widget_show (self->current_panel);
   gtk_stack_set_visible_child_name (self->stack, id);
 
-  /* set the title of the window */
-  gtk_window_set_role (GTK_WINDOW (self), id);
-  gtk_header_bar_set_title (self->panel_headerbar, name);
-
-  title_widget = cc_panel_get_title_widget (CC_PANEL (self->current_panel));
-  gtk_header_bar_set_custom_title (self->panel_headerbar, title_widget);
-
   sidebar_widget = cc_panel_get_sidebar_widget (CC_PANEL (self->current_panel));
   cc_panel_list_add_sidebar_widget (self->panel_list, sidebar_widget);
   /* Ensure we show the panel when the leaflet is folded and a sidebar widget's
@@ -293,7 +260,7 @@ update_list_title (CcWindow *self)
     }
 
   if (title)
-    gtk_header_bar_set_title (self->header, title);
+    adw_window_title_set_title (self->sidebar_title_widget, title);
 
   CC_EXIT;
 }
@@ -386,6 +353,7 @@ update_headerbar_buttons (CcWindow *self)
 
   gtk_widget_set_visible (GTK_WIDGET (self->previous_button), !is_main_view);
   gtk_widget_set_visible (GTK_WIDGET (self->search_button), is_main_view);
+  gtk_search_bar_set_key_capture_widget (self->search_bar, is_main_view ? GTK_WIDGET (self) : NULL);
 
   update_list_title (self);
 
@@ -404,7 +372,6 @@ set_active_panel_from_id (CcWindow     *self,
   g_autofree gchar *name = NULL;
   CcPanelVisibility visibility;
   GtkTreeIter iter;
-  GtkWidget *old_panel;
   CcPanelListView view;
   gboolean activated;
   gboolean found;
@@ -418,7 +385,7 @@ set_active_panel_from_id (CcWindow     *self,
     {
       g_object_set (G_OBJECT (self->current_panel), "parameters", parameters, NULL);
       if (force_moving_to_the_panel || self->previous_list_view == view)
-        show_panel (self);
+        adw_leaflet_navigate (self->main_leaflet, ADW_NAVIGATION_DIRECTION_FORWARD);
       self->previous_list_view = view;
       CC_RETURN (TRUE);
     }
@@ -430,7 +397,16 @@ set_active_panel_from_id (CcWindow     *self,
       CC_RETURN (TRUE);
     }
 
-  old_panel = self->current_panel;
+  if (self->old_panel)
+    gtk_stack_remove (self->stack, g_steal_pointer (&self->old_panel));
+
+  /* old_panel will be removed by the on_stack_transition_running_changed_cb
+   * callback - or, if panels changed before the transition ended, by the code
+   * just above.
+   */
+  self->old_panel = self->current_panel;
+  if (self->old_panel)
+    cc_panel_deactivate (CC_PANEL (self->old_panel));
 
   gtk_tree_model_get (GTK_TREE_MODEL (self->store),
                       &iter,
@@ -454,16 +430,13 @@ set_active_panel_from_id (CcWindow     *self,
     add_current_panel_to_history (self, start_id);
 
   if (force_moving_to_the_panel)
-    show_panel (self);
+    adw_leaflet_navigate (self->main_leaflet, ADW_NAVIGATION_DIRECTION_FORWARD);
 
   g_free (self->current_panel_id);
   self->current_panel_id = g_strdup (start_id);
 
   CC_TRACE_MSG ("Current panel id: %s", start_id);
 
-  if (old_panel)
-    gtk_container_remove (GTK_CONTAINER (self->stack), old_panel);
-
   cc_panel_list_set_active_panel (self->panel_list, start_id);
 
   update_headerbar_buttons (self);
@@ -511,19 +484,31 @@ switch_to_previous_panel (CcWindow *self)
 }
 
 /* Callbacks */
+
+static void
+navigate_action_cb (GtkWidget   *widget,
+                    const gchar *action_name,
+                    GVariant    *parameter)
+{
+  CcWindow *self = CC_WINDOW (widget);
+
+  adw_leaflet_navigate (self->main_leaflet, g_variant_get_int32 (parameter));
+}
+
 static void
 on_main_leaflet_folded_changed_cb (CcWindow *self)
 {
   GtkSelectionMode selection_mode;
+  gboolean folded;
 
   g_assert (CC_IS_WINDOW (self));
 
-  selection_mode = GTK_SELECTION_SINGLE;
-
-  if (hdy_leaflet_get_folded (self->main_leaflet))
-    selection_mode = GTK_SELECTION_NONE;
+  folded = adw_leaflet_get_folded (self->main_leaflet);
 
+  selection_mode = folded ? GTK_SELECTION_NONE : GTK_SELECTION_SINGLE;
   cc_panel_list_set_selection_mode (self->panel_list, selection_mode);
+
+  cc_panel_set_folded (CC_PANEL (self->current_panel), folded);
 }
 
 static void
@@ -546,18 +531,12 @@ search_entry_activate_cb (CcWindow *self)
   gtk_search_bar_set_search_mode (self->search_bar, !changed);
 }
 
-static void
-back_button_clicked_cb (CcWindow *self)
-{
-  show_sidebar (self);
-}
-
 static void
 previous_button_clicked_cb (CcWindow *self)
 {
   g_debug ("Num previous panels? %d", g_queue_get_length (self->previous_panels));
 
-  /* When in search, simply unsed the search mode */
+  /* When in search, simply unset the search mode */
   if (gtk_search_bar_get_search_mode (self->search_bar))
     gtk_search_bar_set_search_mode (self->search_bar, FALSE);
   else
@@ -566,105 +545,39 @@ previous_button_clicked_cb (CcWindow *self)
   update_headerbar_buttons (self);
 }
 
-static void
-gdk_window_set_cb (CcWindow *self)
+static gboolean
+go_back_shortcut_cb (GtkWidget *widget,
+                     GVariant  *args,
+                     gpointer   user_data)
 {
-  GdkWindow *window;
-  g_autofree gchar *str = NULL;
+  g_debug ("Going to previous panel");
+  switch_to_previous_panel (CC_WINDOW (widget));
 
-  if (!GDK_IS_X11_DISPLAY (gdk_display_get_default ()))
-    return;
-
-  window = gtk_widget_get_window (GTK_WIDGET (self));
-
-  if (!window)
-    return;
-
-  str = g_strdup_printf ("%u", (guint) GDK_WINDOW_XID (window));
-  g_setenv ("GNOME_CONTROL_CENTER_XID", str, TRUE);
+  return GDK_EVENT_STOP;
 }
 
 static gboolean
-window_map_event_cb (CcWindow *self,
-                     GdkEvent *event)
+search_shortcut_cb (GtkWidget *widget,
+                    GVariant  *args,
+                    gpointer   user_data)
 {
-  if (event->any.window == gtk_widget_get_window (GTK_WIDGET (self)))
-    {
-      /* If focus ends up in a category icon view one of the items is
-       * immediately selected which looks odd when we are starting up, so
-       * we  explicitly unset the focus here. */
-      gtk_window_set_focus (GTK_WINDOW (self), NULL);
-    }
-
-  return GDK_EVENT_PROPAGATE;
-}
-
-static gboolean
-window_key_press_event_cb (CcWindow    *self,
-                           GdkEventKey *event)
-{
-  GdkModifierType state;
   CcPanelListView view;
-  GdkKeymap *keymap;
-  gboolean retval;
-  gboolean is_rtl;
+  CcWindow *self;
+  gboolean search;
 
-  retval = GDK_EVENT_PROPAGATE;
-  state = event->state;
-  keymap = gdk_keymap_get_for_display (gtk_widget_get_display (GTK_WIDGET (self)));
-  gdk_keymap_add_virtual_modifiers (keymap, &state);
-
-  state = state & gtk_accelerator_get_default_mod_mask ();
-  is_rtl = gtk_widget_get_direction (GTK_WIDGET (self)) == GTK_TEXT_DIR_RTL;
+  self = CC_WINDOW (widget);
   view = cc_panel_list_get_view (self->panel_list);
 
   /* The search only happens when we're in the MAIN view */
-  if (view == CC_PANEL_LIST_MAIN &&
-      gtk_search_bar_handle_event (self->search_bar, (GdkEvent*) event) == GDK_EVENT_STOP)
-    {
-      return GDK_EVENT_STOP;
-    }
+  if (view != CC_PANEL_LIST_MAIN && view != CC_PANEL_LIST_SEARCH)
+    return GDK_EVENT_PROPAGATE;
 
-  if (state == GDK_CONTROL_MASK)
-    {
-      switch (event->keyval)
-        {
-          case GDK_KEY_s:
-          case GDK_KEY_S:
-          case GDK_KEY_f:
-          case GDK_KEY_F:
-            /* The search only happens when we're in the MAIN view */
-            if (view != CC_PANEL_LIST_MAIN &&
-                view != CC_PANEL_LIST_SEARCH)
-              {
-                break;
-              }
+  search = !gtk_search_bar_get_search_mode (self->search_bar);
+  gtk_search_bar_set_search_mode (self->search_bar, search);
+  if (search)
+    gtk_widget_grab_focus (GTK_WIDGET (self->search_entry));
 
-            retval = !gtk_search_bar_get_search_mode (self->search_bar);
-            gtk_search_bar_set_search_mode (self->search_bar, retval);
-            if (retval)
-              gtk_widget_grab_focus (GTK_WIDGET (self->search_entry));
-            retval = GDK_EVENT_STOP;
-            break;
-          case GDK_KEY_Q:
-          case GDK_KEY_q:
-          case GDK_KEY_W:
-          case GDK_KEY_w:
-            gtk_widget_destroy (GTK_WIDGET (self));
-            retval = GDK_EVENT_STOP;
-            break;
-        }
-    }
-  else if ((!is_rtl && state == GDK_MOD1_MASK && event->keyval == GDK_KEY_Left) ||
-           (is_rtl && state == GDK_MOD1_MASK && event->keyval == GDK_KEY_Right) ||
-           event->keyval == GDK_KEY_Back)
-    {
-      g_debug ("Going to previous panel");
-      switch_to_previous_panel (self);
-      retval = GDK_EVENT_STOP;
-    }
-
-  return retval;
+  return GDK_EVENT_STOP;
 }
 
 static void
@@ -676,6 +589,23 @@ on_development_warning_dialog_responded_cb (CcWindow *self)
   gtk_widget_hide (GTK_WIDGET (self->development_warning_dialog));
 }
 
+static void
+on_stack_transition_running_changed_cb (GtkStack   *stack,
+                                        GParamSpec *pspec,
+                                        CcWindow   *self)
+{
+  gboolean transition_running;
+
+  CC_ENTRY;
+
+  transition_running = gtk_stack_get_transition_running (stack);
+
+  if (!transition_running && self->old_panel)
+    gtk_stack_remove (self->stack, g_steal_pointer (&self->old_panel));
+
+  CC_EXIT;
+}
+
 /* CcShell implementation */
 static gboolean
 cc_window_set_active_panel_from_id (CcShell      *shell,
@@ -686,40 +616,6 @@ cc_window_set_active_panel_from_id (CcShell      *shell,
   return set_active_panel_from_id (CC_WINDOW (shell), start_id, parameters, TRUE, TRUE, error);
 }
 
-static void
-cc_window_embed_widget_in_header (CcShell         *shell,
-                                  GtkWidget       *widget,
-                                  GtkPositionType  position)
-{
-  CcWindow *self = CC_WINDOW (shell);
-
-  CC_ENTRY;
-
-  /* add to header */
-  switch (position)
-    {
-    case GTK_POS_RIGHT:
-      gtk_container_add (GTK_CONTAINER (self->top_right_box), widget);
-      break;
-
-    case GTK_POS_LEFT:
-      gtk_container_add (GTK_CONTAINER (self->top_left_box), widget);
-      break;
-
-    case GTK_POS_TOP:
-    case GTK_POS_BOTTOM:
-    default:
-      g_warning ("Invalid position passed");
-      return;
-    }
-
-  g_ptr_array_add (self->custom_widgets, g_object_ref (widget));
-
-  gtk_size_group_add_widget (self->header_sizegroup, widget);
-
-  CC_EXIT;
-}
-
 static GtkWidget *
 cc_window_get_toplevel (CcShell *self)
 {
@@ -730,7 +626,6 @@ static void
 cc_shell_iface_init (CcShellInterface *iface)
 {
   iface->set_active_panel_from_id = cc_window_set_active_panel_from_id;
-  iface->embed_widget_in_header = cc_window_embed_widget_in_header;
   iface->get_toplevel = cc_window_get_toplevel;
 }
 
@@ -747,6 +642,27 @@ cc_window_map (GtkWidget *widget)
     gtk_window_present (GTK_WINDOW (self->development_warning_dialog));
 }
 
+static void
+cc_window_unmap (GtkWidget *widget)
+{
+  CcWindow *self = CC_WINDOW (widget);
+  gboolean maximized;
+  gint height;
+  gint width;
+
+  maximized = gtk_window_is_maximized (GTK_WINDOW (self));
+  gtk_window_get_default_size (GTK_WINDOW (self), &width, &height);
+
+  g_settings_set (self->settings,
+                  "window-state",
+                  "(iib)",
+                  width,
+                  height,
+                  maximized);
+
+  GTK_WIDGET_CLASS (cc_window_parent_class)->unmap (widget);
+}
+
 /* GObject Implementation */
 static void
 cc_window_get_property (GObject    *object,
@@ -766,6 +682,10 @@ cc_window_get_property (GObject    *object,
       g_value_set_object (value, self->store);
       break;
 
+    case PROP_FOLDED:
+      g_value_set_boolean (value, self->folded);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
     }
@@ -790,6 +710,10 @@ cc_window_set_property (GObject      *object,
       self->store = g_value_dup_object (value);
       break;
 
+    case PROP_FOLDED:
+      self->folded = g_value_get_boolean (value);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
     }
@@ -801,6 +725,8 @@ cc_window_constructed (GObject *object)
   CcWindow *self = CC_WINDOW (object);
   g_autofree char *id = NULL;
 
+  load_window_state (self);
+
   /* Add the panels */
   setup_model (self);
 
@@ -818,7 +744,8 @@ cc_window_constructed (GObject *object)
                             self);
 
   update_headerbar_buttons (self);
-  show_sidebar (self);
+  adw_leaflet_set_visible_child (self->main_leaflet,
+                                 GTK_WIDGET (self->sidebar_box));
 
   G_OBJECT_CLASS (cc_window_parent_class)->constructed (object);
 }
@@ -829,7 +756,6 @@ cc_window_dispose (GObject *object)
   CcWindow *self = CC_WINDOW (object);
 
   g_clear_pointer (&self->current_panel_id, g_free);
-  g_clear_pointer (&self->custom_widgets, g_ptr_array_unref);
   g_clear_object (&self->store);
   g_clear_object (&self->active_panel);
 
@@ -865,6 +791,7 @@ cc_window_class_init (CcWindowClass *klass)
   object_class->finalize = cc_window_finalize;
 
   widget_class->map = cc_window_map;
+  widget_class->unmap = cc_window_unmap;
 
   g_object_class_override_property (object_class, PROP_ACTIVE_PANEL, "active-panel");
 
@@ -876,35 +803,47 @@ cc_window_class_init (CcWindowClass *klass)
                                                         CC_TYPE_SHELL_MODEL,
                                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS));
 
-  gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/ControlCenter/gtk/cc-window.ui");
+  g_object_class_install_property (object_class,
+                                   PROP_FOLDED,
+                                   g_param_spec_boolean ("folded",
+                                                         "Folded",
+                                                         "Whether the window is foled",
+                                                         FALSE,
+                                                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+  gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/Settings/gtk/cc-window.ui");
 
-  gtk_widget_class_bind_template_child (widget_class, CcWindow, back_revealer);
   gtk_widget_class_bind_template_child (widget_class, CcWindow, development_warning_dialog);
   gtk_widget_class_bind_template_child (widget_class, CcWindow, header);
-  gtk_widget_class_bind_template_child (widget_class, CcWindow, header_box);
-  gtk_widget_class_bind_template_child (widget_class, CcWindow, header_sizegroup);
   gtk_widget_class_bind_template_child (widget_class, CcWindow, main_leaflet);
-  gtk_widget_class_bind_template_child (widget_class, CcWindow, panel_headerbar);
   gtk_widget_class_bind_template_child (widget_class, CcWindow, panel_list);
   gtk_widget_class_bind_template_child (widget_class, CcWindow, previous_button);
   gtk_widget_class_bind_template_child (widget_class, CcWindow, search_bar);
   gtk_widget_class_bind_template_child (widget_class, CcWindow, search_button);
   gtk_widget_class_bind_template_child (widget_class, CcWindow, search_entry);
   gtk_widget_class_bind_template_child (widget_class, CcWindow, sidebar_box);
+  gtk_widget_class_bind_template_child (widget_class, CcWindow, sidebar_title_widget);
   gtk_widget_class_bind_template_child (widget_class, CcWindow, stack);
-  gtk_widget_class_bind_template_child (widget_class, CcWindow, top_left_box);
-  gtk_widget_class_bind_template_child (widget_class, CcWindow, top_right_box);
 
-  gtk_widget_class_bind_template_callback (widget_class, back_button_clicked_cb);
-  gtk_widget_class_bind_template_callback (widget_class, gdk_window_set_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_main_leaflet_folded_changed_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_development_warning_dialog_responded_cb);
+  gtk_widget_class_bind_template_callback (widget_class, on_stack_transition_running_changed_cb);
   gtk_widget_class_bind_template_callback (widget_class, previous_button_clicked_cb);
   gtk_widget_class_bind_template_callback (widget_class, search_entry_activate_cb);
   gtk_widget_class_bind_template_callback (widget_class, show_panel_cb);
   gtk_widget_class_bind_template_callback (widget_class, update_list_title);
-  gtk_widget_class_bind_template_callback (widget_class, window_key_press_event_cb);
-  gtk_widget_class_bind_template_callback (widget_class, window_map_event_cb);
+
+  gtk_widget_class_add_binding (widget_class, GDK_KEY_Left, GDK_ALT_MASK, go_back_shortcut_cb, NULL);
+  gtk_widget_class_add_binding (widget_class, GDK_KEY_s, GDK_CONTROL_MASK, search_shortcut_cb, NULL);
+  gtk_widget_class_add_binding (widget_class, GDK_KEY_S, GDK_CONTROL_MASK, search_shortcut_cb, NULL);
+  gtk_widget_class_add_binding (widget_class, GDK_KEY_f, GDK_CONTROL_MASK, search_shortcut_cb, NULL);
+  gtk_widget_class_add_binding (widget_class, GDK_KEY_F, GDK_CONTROL_MASK, search_shortcut_cb, NULL);
+  gtk_widget_class_add_binding_action (widget_class, GDK_KEY_q, GDK_CONTROL_MASK, "window.close", NULL);
+  gtk_widget_class_add_binding_action (widget_class, GDK_KEY_Q, GDK_CONTROL_MASK, "window.close", NULL);
+  gtk_widget_class_add_binding_action (widget_class, GDK_KEY_w, GDK_CONTROL_MASK, "window.close", NULL);
+  gtk_widget_class_add_binding_action (widget_class, GDK_KEY_W, GDK_CONTROL_MASK, "window.close", NULL);
+
+  gtk_widget_class_install_action (widget_class, "window.navigate", "i", navigate_action_cb);
 
   g_type_ensure (CC_TYPE_PANEL_LIST);
 }
@@ -914,13 +853,16 @@ cc_window_init (CcWindow *self)
 {
   gtk_widget_init_template (GTK_WIDGET (self));
 
-  gtk_widget_add_events (GTK_WIDGET (self), GDK_BUTTON_RELEASE_MASK);
-
-  self->settings = g_settings_new ("org.gnome.ControlCenter");
-  self->custom_widgets = g_ptr_array_new_with_free_func ((GDestroyNotify) g_object_unref);
+  self->settings = g_settings_new ("org.gnome.Settings");
   self->previous_panels = g_queue_new ();
   self->previous_list_view = cc_panel_list_get_view (self->panel_list);
 
+  g_object_bind_property (self->main_leaflet,
+                          "folded",
+                          self,
+                          "folded",
+                          G_BINDING_SYNC_CREATE);
+
   /* Add a custom CSS class on development builds */
   if (in_flatpak_sandbox ())
     gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (self)), "devel");
@@ -937,7 +879,6 @@ cc_window_new (GtkApplication *application,
                        "resizable", TRUE,
                        "title", _("Settings"),
                        "icon-name", DEFAULT_WINDOW_ICON_NAME,
-                       "window-position", GTK_WIN_POS_CENTER,
                        "show-menubar", FALSE,
                        "model", model,
                        NULL);
@@ -948,6 +889,6 @@ cc_window_set_search_item (CcWindow   *center,
                            const char *search)
 {
   gtk_search_bar_set_search_mode (center->search_bar, TRUE);
-  gtk_entry_set_text (GTK_ENTRY (center->search_entry), search);
+  gtk_editable_set_text (GTK_EDITABLE (center->search_entry), search);
   gtk_editable_set_position (GTK_EDITABLE (center->search_entry), -1);
 }
